from typing import List

import intermake_qt.utilities.interfaces
from mhelper import SwitchError
from mhelper_qt import exceptToGui, exqtSlot
from PyQt5.QtCore import QCoreApplication, Qt, QTimer
from PyQt5.QtWidgets import QMainWindow, QMessageBox

import intermake
import bio42
import neocommand
import intermake_qt

from bio42_gui.forms.designer import frm_b42main_designer
from bio42_gui.forms.frm_b42_base import AbstractB42SubWindow
from bio42_gui.forms.frm_neo4j_wizard import FrmNeo4jWizard


class FrmB42Main( QMainWindow, intermake_qt.utilities.interfaces.IGuiMainWindow ):
    """
    Main window
    """
    
    
    @exceptToGui()
    def __init__( self, controller ) -> None:
        """
        CONSTRUCTOR
        """
        from bio42_gui import B42GuiController
        
        # QT stuff
        QCoreApplication.setAttribute( Qt.AA_DontUseNativeMenuBar )
        QMainWindow.__init__( self )
        self.ui = frm_b42main_designer.Ui_MainWindow()
        self.ui.setupUi( self )
        self.setWindowTitle( "B42" )
        self.ui.PAGER_MAIN.setCurrentIndex( 0 )
        
        self.child_windows: List[AbstractB42SubWindow] = []
        self.ui.LBL_INTRO.linkActivated[str].connect( self.on_lbl_intro_link_activated )
        
        self.timer = QTimer( self )
        self.timer.timeout.connect( self.flash )
        self.flash_stage = 0
        self.flash_count = 0
        self.flash_button = ()
        self.flash_page = None
        
        self.controller: B42GuiController = controller
    
    
    def flash( self ):
        DELAY_1 = 5
        DELAY_2 = 5
        DELAY_3 = 10
        DELAY_4 = 10
        
        if self.flash_stage == 1:
            # Setup
            self.flash_count = DELAY_1
            self.flash_stage += 1
        elif self.flash_stage == 2:
            # Delay
            self.flash_count -= 1
            
            if self.flash_count == 0:
                self.ui.PAGER_MAIN.setCurrentWidget( self.flash_page )
                self.flash_count = DELAY_2
                self.flash_stage += 1
        elif self.flash_stage == 3:
            # Delay
            self.flash_count -= 1
            
            if self.flash_count == 0:
                self.flash_count = DELAY_3
                self.flash_stage += 1
        elif self.flash_stage == 4:
            # Flash button
            self.flash_count -= 1
            
            if self.flash_count == 0:
                self.flash_count = DELAY_4
                self.flash_stage += 1
                ss = ""
            elif self.flash_count % 2 == 0:
                ss = "border-bottom: 2px solid yellow;"
            else:
                ss = "border-bottom: 2px solid black;"
            
            for button in self.flash_button:
                button.setStyleSheet( ss )
        elif self.flash_stage == 5:
            self.flash_count -= 1
            
            if self.flash_count == 0:
                self.timer.stop()
                self.ui.PAGER_MAIN.setCurrentIndex( 0 )
                self.on_lbl_intro_link_activated( "" )
    
    
    def on_lbl_intro_link_activated( self, _ ):
        from bio42_gui.forms.frm_getting_started import FrmGettingStarted as f
        v = f.request( self )
        
        if v == f.O_CREATE_DATABASE:
            self.highlight( self.ui.PAG_ENDPOINTS, self.ui.BTN_ENDPOINT_TRANSFER )
        elif v == f.O_CREATE_PARCEL:
            self.highlight( self.ui.PAG_ENDPOINTS, self.ui.BTN_ENDPOINT_PACKAGE )
        elif v == f.O_EXPLORE_DATA:
            self.highlight( self.ui.PAG_QUERY, self.ui.BTN_QUERY_INBUILT )
        elif v == f.O_OPEN_CONNECTION:
            self.highlight( self.ui.PAG_ENDPOINTS, self.ui.BTN_WIZARD, self.ui.BTN_ENDPOINT_DATABASE )
        elif v == f.O_PUT_DATA_IN:
            self.highlight( self.ui.PAG_IMPORT, self.ui.BTN_IMPORT_TAXONOMY, self.ui.BTN_IMPORT_ANNOTATIONS, self.ui.BTN_IMPORT_CSV, self.ui.BTN_IMPORT_ONTOLOGY, self.ui.BTN_IMPORT_SEQUENCES, self.ui.BTN_IMPORT_SIMILARITIES )
        elif v is not None:
            raise SwitchError( "f.request", v )
    
    
    def highlight( self, page, *button ):
        self.flash_page = page
        self.flash_button = button
        self.flash_stage = 1
        self.timer.start( 100 )
    
    
    def command_completed( self, result: intermake.Result ) -> None:
        for window in self.child_windows:
            window.on_plugin_completed( result )
    
    
    def return_to_console( self ) -> bool:
        pass
    
    
    @exqtSlot()
    def on_BTN_ENDPOINT_DATABASE_clicked( self ) -> None:
        """
        Signal handler:
        """
        self.__request( neocommand.open_database )
    
    
    @exqtSlot()
    def on_BTN_ENDPOINT_PACKAGE_clicked( self ) -> None:
        """
        Signal handler:
        """
        self.__request( neocommand.open_parcel )
    
    
    @exqtSlot()
    def on_BTN_ENDPOINT_DISCONNECT_clicked( self ) -> None:
        """
        Signal handler:
        """
        from .frm_endpoint_list import FrmEndpointList
        FrmEndpointList.request( self )
    
    
    @exqtSlot()
    def on_BTN_IMPORT_SEQUENCES_clicked( self ) -> None:
        """
        Signal handler:
        """
        self.__request( bio42.import_sequences )
    
    
    @exqtSlot()
    def on_BTN_IMPORT_SIMILARITIES_clicked( self ) -> None:
        """
        Signal handler:
        """
        self.__request( bio42.import_blast )
    
    
    @exqtSlot()
    def on_BTN_IMPORT_ANNOTATIONS_clicked( self ) -> None:
        """
        Signal handler:
        """
        self.__request( bio42.import_annotations )
    
    
    @exqtSlot()
    def on_BTN_IMPORT_ONTOLOGY_clicked( self ) -> None:
        """
        Signal handler:
        """
        self.__request( bio42.import_go )
    
    
    @exqtSlot()
    def on_BTN_IMPORT_TAXONOMY_clicked( self ) -> None:
        """
        Signal handler:
        """
        self.__request( bio42.import_taxonomy )
    
    
    @exqtSlot()
    def on_BTN_IMPORT_CSV_clicked( self ) -> None:
        """
        Signal handler:
        """
        from bio42_gui.forms.frm_import_csv import FrmImportCsv
        FrmImportCsv.request( self )
    
    
    @exqtSlot()
    def on_BTN_QUERY_INBUILT_clicked( self ) -> None:
        """
        Signal handler:
        """
        from bio42_gui.forms.frm_query_browser import FrmQueryBrowser
        FrmQueryBrowser.request( self )
    
    
    @exqtSlot()
    def on_BTN_SETTINGS_clicked( self ) -> None:
        """
        Signal handler:
        """
        self.__request( intermake.commands.configure )
    
    
    @exqtSlot()
    def on_BTN_OPTIMISE_clicked( self ) -> None:
        """
        Signal handler:
        """
        self.__request( neocommand.apply_schema )
    
    
    @exqtSlot()
    def on_BTN_EXPORT_GEXF_clicked( self ) -> None:
        """
        Signal handler:
        """
        QMessageBox.information( self,
                                 "GEXF-Export",
                                 "GEXF currently can't be exported directly from the main screen. Please create a `file - gexf` endpoint and `transfer` to it for now." )
    
    
    @exqtSlot()
    def on_BTN_EXPORT_CSV_clicked( self ) -> None:
        """
        Signal handler:
        """
        self.__request( neocommand.export_edgelist )
    
    
    @exqtSlot()
    def on_BTN_EXPORT_HTML_clicked( self ) -> None:
        """
        Signal handler:
        """
        self.__request( neocommand.export_js )
    
    
    @exqtSlot()
    def on_BTN_QUERY_PICKLE_clicked( self ) -> None:
        """
        Signal handler:
        """
        self.__request( neocommand.export_binary )
    
    
    @exqtSlot()
    def on_BTN_WIZARD_clicked( self ) -> None:
        """
        Signal handler:
        """
        FrmNeo4jWizard.request( self )
    
    
    @exqtSlot()
    def on_BTN_DB_STATUS_clicked( self ) -> None:
        """
        Signal handler:
        """
        from bio42_gui.forms.frm_database import FrmDatabase
        FrmDatabase.request( self )
    
    
    @exqtSlot()
    def on_BTN_ENDPOINT_MGRAPH_clicked( self ) -> None:
        """
        Signal handler:
        """
        self.__request( neocommand.open_file )
    
    
    @exqtSlot()
    def on_BTN_ENDPOINT_TRANSFER_clicked( self ) -> None:
        """
        Signal handler:
        """
        from bio42_gui.forms.frm_transfer import FrmTransfer
        FrmTransfer.request( self )
    
    
    @exqtSlot()
    def on_BTN_INTERMAKE_clicked( self ) -> None:
        """
        Signal handler:
        """
        intermake_qt.show_basic_window( self )
    
    
    @exqtSlot()
    def on_BTN_HELP_clicked( self ) -> None:
        """
        Signal handler:
        """
        pass
    
    
    def __request( self, command ):
        return self.controller.acquire( command, window = self, confirm = True ).run()



import csv
from typing import Dict, List, Optional
import intermake
from mhelper import EFileMode, isFilename
import neocommand

from bio42.application import app
from bio42 import schema as b42schema


_EXT_CSV = ".csv"
_EXT_TSV = ".tsv"


@app.command( folder = "import" )
def import_annotations( destination: neocommand.Destination,
                        file_name: isFilename[EFileMode.READ, _EXT_CSV, _EXT_TSV],
                        create_classes: bool = False,
                        viable_columns: Optional[List[str]] = None,
                        name_delimiter: str = "|",
                        uid_delimiter: str = "|",
                        pad: bool = False
                        ) -> None:
    """
    Imports gene annotations from a CSV or TSV file.
    
    The following columns of the source file are used.

    =============== ===========================================================
    Column          Meaning    
    =============== ===========================================================
    uid             the accession of the gene sequence
    xxx             a column containing accession(s) of the annotation terms.
                    The column must be named after a valid annotation label.
                    
                    * A node schema must exist defining nodes with this label
                    * An edge schema must exist defining edges from nodes with
                      this label to nodes with the 'Sequence' label. The label
                      of the edges themselves are ignored.
                    
                    Valid inbuilt values include `Go`, `Pfam`, `Ipr` and `Kegg`.
    xxx.name        a column containing the name(s) of the annotation terms
                    These columns are optional, and are only used if the
                    `create_classes` argument is passed.  
    =============== ===========================================================
    
    All columns must be valid, though columns can be ignored by passing the
    `viable_columns` argument.
    
    :param destination:        Where to send the parsed data to. 
    :param file_name:       File to read in.
    :param create_classes:  Whether to create the annotation classes.
    :param viable_columns:  Which columns to accept.
                            If not specified then all columns are assumed.
                            The `uid` column is implicit and need not be specified.
    :param name_delimiter:  Delimiter for multiple annotation UIDs in the same column.
    :param uid_delimiter:   Delimiter for multiple annotation names in the same column.
                            This is only used if `create_classes` is set.
    :param pad:             When set, missing columns are treated as empty.
                            This allows jagged input to be imported without error.
    """
    NAME_KEY_SUFFIX = ".name"
    schema = neocommand.get_core().schema
    gene_node_schema = b42schema.Sequence
    
    #
    # Determine the input delimiter
    #
    if file_name.endswith( ".csv" ):
        delimiter = ","
    elif file_name.endswith( ".tsv" ):
        delimiter = "\t"
    else:
        raise ValueError( "File must end with <.csv> or <.tsv> but «{0}» does not.".format( file_name ) )
    
    #
    # Open the destination for writing
    #
    with destination.open_writer() as writer:
        #
        # Open the source for reading
        #
        with open( file_name, "r" ) as file_in:
            reader = csv.reader( file_in, delimiter = delimiter )
            
            #
            # Map out the column headers
            #
            col_names = next( reader )
            
            col_anno_indices: Dict[str, int] = { }  # ------------------------- Maps normal column names to their indices
            col_anno_edge_schema: Dict[str, neocommand.EdgeSchema] = { }  # --- Maps normal column names to their edge schema 
            col_anno_node_schema: Dict[str, neocommand.NodeSchema] = { }  # --- Maps normal column names to their node schema
            col_gene_id: int = None  # ---------------------------------------- Maps the special UID column to its index
            col_anno_names: Dict[str, int] = { }  # --------------------------- Maps normal column names to indices of their counterpart name columns (these have the same name but ending ".name")
            
            for i, col_name in enumerate( col_names ):
                if col_name == "uid":
                    col_gene_id = i
                else:
                    if viable_columns is not None and col_name not in viable_columns:
                        continue
                    
                    if col_name.endswith( NAME_KEY_SUFFIX ):
                        col_anno_names[col_name[:len( NAME_KEY_SUFFIX )]] = i
                    else:
                        col_anno_indices[col_name] = i
                        
                        for anno_node_schema in schema.node_schema:
                            if col_name == anno_node_schema.label:
                                col_anno_node_schema[col_name] = anno_node_schema
                                
                                for edge_schema in schema.edge_schema:
                                    if edge_schema.start_label == anno_node_schema and edge_schema.end_label == gene_node_schema:
                                        col_anno_edge_schema[col_name] = edge_schema
                                        break
                                else:
                                    raise ValueError( "Cannot read the file «{}» because the «{}» "
                                                      "column is not named after a valid schema: "
                                                      "there is no defined schema for edges from «{}» to «{}».".format( file_name, col_name, anno_node_schema, gene_node_schema ) )
                        
                        else:
                            raise ValueError( "Cannot read the file «{}» because the «{}» "
                                              "column is not named after a valid schema: "
                                              "there is no defined schema for nodes named «{}».".format( file_name, col_name, col_name ) )
            
            #
            # Check the headers are valid
            #
            
            # each `col_class_name` should have an associated `col_index`
            for key in col_anno_names:
                if key not in col_anno_indices:
                    raise ValueError( "Cannot read the file «{0}» because whilst there is a <.name> column for «{1}» (<{1}.name>), there is no UID column («{1}»). The columns present are «{2}».".format( file_name, key, col_names ) )
            
            # There must be a UID column
            if col_gene_id is None:
                raise ValueError( "Cannot read the file «{0}» because none of the mandatory <uid> column does not exist. The columns present are «{1}».".format( file_name, col_names ) )
            
            # There must be at least one annotation column
            if not col_anno_indices:
                raise ValueError( "Cannot read the file «{0}» because it doesn't contain any other accepted columns. The columns present are «{1}».".format( file_name, col_names ) )
            
            #
            # Display the mapping to the user
            #
            anno_names: Dict[str, Dict[str, str]] = { }  # Maps column names to dictionaries mapping annotation UIDs to names
            
            for col_name, col_index in col_anno_indices.items():
                name = col_anno_names.get( col_name )
                anno_names[col_name] = { }
                
                if name is not None:
                    print( "«{}» in column «{}» with name in column «{}».".format( col_name, col_index, name ) )
                else:
                    print( "«{}» in column «{}».".format( col_name, col_index ) )
            
            #
            # Create the actual edges
            #
            for i, row in intermake.pr.pr_enumerate( reader, "Creating edges" ):
                gene_uid = row[col_gene_id]
                
                for col_name, col_index in col_anno_indices.items():
                    annotation_class_name = col_anno_names.get( col_name )
                    col_edge = col_anno_edge_schema.get( col_name )
                    
                    __apply_annotation( writer = writer,
                                        gene_uid = gene_uid,
                                        row = row,
                                        anno_names = anno_names[col_name],
                                        col_anno_uid = col_index,
                                        col_anno_name = annotation_class_name,
                                        anno_edge_label = col_edge,
                                        anno_name_delimiter = name_delimiter,
                                        anno_uid_delimiter = uid_delimiter,
                                        pad = pad )
            
            #
            # Create the classes
            #
            if create_classes:
                for col_name, name_dict in anno_names.items():
                    with intermake.pr.pr_action( "Creating «{}» nodes".format( col_name ), len( name_dict ) ) as action:
                        for anno_uid, anno_name in name_dict.items():
                            if name:
                                data = { "name": name }
                            else:
                                data = { }
                            
                            action.increment()
                            writer.create_node( label = col_name,
                                                uid = anno_uid,
                                                properties = data )


def __apply_annotation( writer: neocommand.Writer,
                        gene_uid: str,
                        row: List[str],
                        anno_names: Dict[str, Optional[str]],
                        col_anno_uid: int,
                        col_anno_name: Optional[int],
                        anno_edge_label: neocommand.EdgeSchema,
                        anno_name_delimiter,
                        anno_uid_delimiter,
                        pad ) -> None:
    """
    Adds an edge from a column.
    
    :param writer:                      Where to send edges to.
    :param gene_uid:                    UID of the gene.
    :param row:                         Contents of the CSV row.
    :param anno_names:                  Receives annotation UIDs and names.
    :param col_anno_uid:                Index of column containing the annotation UID(s).
    :param col_anno_name:               Index of column containing the annotation name(s)
    :param anno_edge_label:             Label to give the created edges. 
    :param anno_name_delimiter:         Used to split `col_annotation_name`.
    :param anno_uid_delimiter:          Used to split `col_annotation_uid`
    :param pad:                         Whether to ignore the case when columns are missing.
    """
    if col_anno_uid < 0 or col_anno_uid >= len( row ):
        if pad:
            return
        raise KeyError( "Row does not have column «{}», which is specified as the UID, and the <pad> argument is <False>: «{}».".format( col_anno_uid, row ) )
    
    uid_text = row[col_anno_uid]
    
    if col_anno_name is None:
        name_texts = None
    else:
        name_text = row[col_anno_name]
        name_texts = name_text.split( anno_name_delimiter )
    
    for i, row_text in enumerate( uid_text.split( anno_uid_delimiter ) ):
        if row_text:
            uid = row_text
            
            if name_texts is not None:
                name = name_texts[i]
            else:
                name = None
            
            anno_names[uid] = name
            
            writer.create_edge( label = anno_edge_label,
                                start_uid = uid,
                                end_uid = gene_uid,
                                properties = { } )



from mhelper import io_helper
from mhelper_qt import exqtSlot, qt_gui_helper
from PyQt5.QtWidgets import QMessageBox, QWidget
from bio42 import Application

import os.path
import webbrowser
import intermake
import neocommand

from bio42_gui.forms.designer import frm_neo4j_wizard_designer
from bio42_gui.forms.frm_b42_base import AbstractB42SubWindow


class FrmNeo4jWizard( AbstractB42SubWindow ):
    """
    This wizard-style dialogue guides the user through installing and connecting to Neo4j.
    
    Uses the `open_database` command.
    """
    
    def __init__( self, parent: QWidget ):
        """
        CONSTRUCTOR
        """
        super().__init__( parent )
        self.ui = frm_neo4j_wizard_designer.Ui_Dialog( self )
        self.setWindowTitle( "Wizard" )
        self.ui.LBL_FINISHED.setVisible( False )
        self.locked_index = 0
        self.ui.TAB_MAIN.setCurrentIndex( 0 )
        self.ui.TAB_MAIN.currentChanged[int].connect( self.__on_TAB_MAIN_currentChanged )
        self.ep = None
    
    
    def __on_TAB_MAIN_currentChanged( self, index ):
        if index != self.locked_index:
            self.ui.TAB_MAIN.setCurrentIndex( self.locked_index )
    
    
    def next_page( self ):
        self.locked_index += 1
        self.ui.TAB_MAIN.setCurrentIndex( self.locked_index )
    
    
    def alert( self, text, det = None ):
        msg = QMessageBox( self )
        msg.setText( "Cannot continue" )
        msg.setInformativeText( text )
        msg.setDetailedText( det )
        msg.setIcon( QMessageBox.Critical )
        msg.exec_()
    
    
    @exqtSlot()
    def on_BTN_BEGIN_clicked( self ) -> None:
        """
        Signal handler:
        """
        for ep in neocommand.get_core().endpoint_manager.user_endpoints:
            if ep.name == "neo4j":
                self.alert( "You already have an endpoint named 'neo4j'. Please close the wizard and «disconnect» that before continuing." )
                return
        
        self.next_page()
    
    
    @exqtSlot()
    def on_BTN_OPEN_DOWNLOAD_clicked( self ) -> None:
        """
        Signal handler:
        """
        webbrowser.open( self.ui.BTN_OPEN_DOWNLOAD.text() )
    
    
    @exqtSlot()
    def on_BTN_BROWSE_DOWNLOAD_clicked( self ) -> None:
        """
        Signal handler:
        """
        qt_gui_helper.browse_dir_on_textbox( self.ui.TXT_DOWNLOAD )
    
    
    @exqtSlot()
    def on_BTN_VERIFY_DOWNLOAD_clicked( self ) -> None:
        """
        Signal handler: Stage 1 - verify
        """
        txt = self.ui.TXT_DOWNLOAD.text()
        
        if not os.path.isdir( txt ):
            self.alert( "'{}' is not a directory.".format( txt ) )
            return
        
        txt = os.path.join( txt, "bin", "neo4j" )
        
        if io_helper.get_system() == io_helper.ESystem.WINDOWS:
            txt += ".exe"
        
        if not os.path.isfile( txt ):
            self.alert( "'{}' does not exist.".format( txt ) )
            return
        
        self.next_page()
    
    
    @exqtSlot()
    def on_BTN_START_clicked( self ) -> None:
        """
        Signal handler: Stage 2 - start
        """
        ep = self.mk_endpoint( directory = self.ui.TXT_DOWNLOAD.text() )
        intermake.acquire( _start_neo4j,
                           window = self,
                           auto_close = True,
                           ).run( ep ).listen( self.__start_neo4j_completed )
    
    
    def __start_neo4j_completed( self, result: intermake.Result ):
        assert result.command == _start_neo4j
        
        if result.is_error:
            self.alert( "Couldn't start Neo4j.", str( result.exception ) )
        else:
            self.next_page()
    
    
    def __finish_wizard( self ):
        self.ui.BTN_OPTIMISE.setEnabled( False )
        self.ui.BTN_DONT_OPTIMISE.setEnabled( False )
        self.ui.LBL_FINISHED.setVisible( True )
    
    
    @exqtSlot()
    def on_BTN_OPEN_LOGON_clicked( self ) -> None:
        """
        Signal handler:
        """
        webbrowser.open( self.ui.BTN_OPEN_LOGON.text() )
    
    
    @exqtSlot()
    def on_BTN_VERIFY_LOGON_clicked( self ) -> None:
        """
        Signal handler:
        """
        ep = self.mk_endpoint( directory = self.ui.TXT_DOWNLOAD.text(),
                               user_name = self.ui.TXT_USERNAME.text(),
                               password = self.ui.TXT_PASSWORD.text() )
        
        intermake.acquire( neocommand.test_connection,
                           window = self,
                           auto_close = True,
                           ).run( ep ).listen( self.__test_connection_completed )
    
    
    def __test_connection_completed( self, result: intermake.Result ):
        if result.is_error:
            self.alert( "Couldn't logon to Neo4j due to a runtime exception.", str( result.exception ) )
        else:
            r: neocommand.Neo4jStatus = result.result
            
            if not r.is_running:
                self.alert( "Neo4j is not running.", r.error )
            elif not r.is_connected:
                self.alert( "Couldn't logon to Neo4j due to an error.", r.error )
            else:
                self.next_page()
    
    
    def mk_endpoint( self, *, name = "neo4j", user_name = "neo4j", password = "neo4j", directory = None, keyring = False ):
        return neocommand.DatabaseEndpoint( name = name,
                                            driver = "neo4jv1",
                                            remote_address = "127.0.0.1",
                                            user_name = user_name,
                                            password = password,
                                            directory = directory,
                                            is_unix = io_helper.get_system() != io_helper.ESystem.WINDOWS,
                                            port = "7474",
                                            keyring = keyring )
    
    
    @exqtSlot()
    def on_BTN_SAVE_clicked( self ) -> None:
        """
        Signal handler:
        """
        self.ep = self.mk_endpoint( directory = self.ui.TXT_DOWNLOAD.text(),
                                    user_name = self.ui.TXT_USERNAME.text(),
                                    password = self.ui.TXT_PASSWORD.text(),
                                    keyring = True )
        
        neocommand.get_core().endpoint_manager.add_user_endpoint( self.ep )
        self.next_page()
    
    
    @exqtSlot()
    def on_BTN_OPTIMISE_clicked( self ) -> None:
        """
        Signal handler:
        """
        intermake.acquire( command = neocommand.apply_schema,
                           window = self
                           ).run( self.ep ).listen( self.__applied_schema )
    
    
    def __applied_schema( self, result: intermake.Result ):
        if result.is_error:
            self.alert( "Failed to apply schema due to a runtime exception.", str( result.exception ) )
        else:
            self.__finish_wizard()
    
    
    @exqtSlot()
    def on_BTN_DONT_OPTIMISE_clicked( self ) -> None:
        """
        Signal handler:
        """
        self.__finish_wizard()


@Application.INSTANCE.command( visibility = intermake.visibilities.INTERNAL )
def _start_neo4j( endpoint: neocommand.DatabaseEndpoint ) -> None:
    """
    Starts the Neo4j server.
    This is a private Command.
    
    :param endpoint:   Endpoint 
    """
    neocommand.control_server( endpoint, neocommand.ENeo4jCommand.START )
    out = neocommand.control_server( endpoint, neocommand.ENeo4jCommand.STATUS )
    if not "Neo4j is running" in out:
        raise ValueError( "Failed to start Neo4j. Neo4j reported the following: {}".format( out ) )



"""
Subpackage containing the the BIO42 commands exposed to the user in the GUI/CLI.
"""

from .downloads import download_accessions, download_annotations, download_taxonomy, download_ontology
from .parsers import import_annotations, import_blast, import_csv, import_go, import_taxonomy, import_sequences, ESequenceFlags


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/martinrusilowicz/work/apps/bio42/bio42_gui/forms/designer/frm_getting_started_designer.ui'
#
# Created by: PyQt5 UI code generator 5.9
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Dialog(object):
    def __init__(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(706, 501)
        self.verticalLayout = QtWidgets.QVBoxLayout(Dialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QtWidgets.QLabel(Dialog)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.BTN_CREATE_PARCEL = QtWidgets.QCommandLinkButton(Dialog)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/bio42/parcel.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_CREATE_PARCEL.setIcon(icon)
        self.BTN_CREATE_PARCEL.setIconSize(QtCore.QSize(48, 48))
        self.BTN_CREATE_PARCEL.setObjectName("BTN_CREATE_PARCEL")
        self.verticalLayout_2.addWidget(self.BTN_CREATE_PARCEL)
        self.BTN_PUT_DATA_IN = QtWidgets.QCommandLinkButton(Dialog)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/bio42/taxonomy.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_PUT_DATA_IN.setIcon(icon1)
        self.BTN_PUT_DATA_IN.setIconSize(QtCore.QSize(48, 48))
        self.BTN_PUT_DATA_IN.setObjectName("BTN_PUT_DATA_IN")
        self.verticalLayout_2.addWidget(self.BTN_PUT_DATA_IN)
        self.BTN_OPEN_CONNECTION = QtWidgets.QCommandLinkButton(Dialog)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/bio42/wizard.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_OPEN_CONNECTION.setIcon(icon2)
        self.BTN_OPEN_CONNECTION.setIconSize(QtCore.QSize(48, 48))
        self.BTN_OPEN_CONNECTION.setObjectName("BTN_OPEN_CONNECTION")
        self.verticalLayout_2.addWidget(self.BTN_OPEN_CONNECTION)
        self.BTN_CREATE_DATABASE = QtWidgets.QCommandLinkButton(Dialog)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/bio42/send.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_CREATE_DATABASE.setIcon(icon3)
        self.BTN_CREATE_DATABASE.setIconSize(QtCore.QSize(48, 48))
        self.BTN_CREATE_DATABASE.setObjectName("BTN_CREATE_DATABASE")
        self.verticalLayout_2.addWidget(self.BTN_CREATE_DATABASE)
        self.BTN_EXPLORE_DATA = QtWidgets.QCommandLinkButton(Dialog)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/bio42/script.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_EXPLORE_DATA.setIcon(icon4)
        self.BTN_EXPLORE_DATA.setIconSize(QtCore.QSize(48, 48))
        self.BTN_EXPLORE_DATA.setObjectName("BTN_EXPLORE_DATA")
        self.verticalLayout_2.addWidget(self.BTN_EXPLORE_DATA)
        self.verticalLayout.addLayout(self.verticalLayout_2)
        spacerItem = QtWidgets.QSpacerItem(20, 92, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Getting started"))
        self.label.setText(_translate("Dialog", "Things to do"))
        self.label.setProperty("style", _translate("Dialog", "title"))
        self.BTN_CREATE_PARCEL.setText(_translate("Dialog", "Create a parcel to put my data in"))
        self.BTN_PUT_DATA_IN.setText(_translate("Dialog", "Put some data in my parcel"))
        self.BTN_OPEN_CONNECTION.setText(_translate("Dialog", "Open a connection to Neo4j"))
        self.BTN_CREATE_DATABASE.setText(_translate("Dialog", "Create my database"))
        self.BTN_EXPLORE_DATA.setText(_translate("Dialog", "Explore my data"))





from typing import Iterable, cast
from mhelper_qt import ComboBoxWrapper, exqtSlot
from PyQt5.QtWidgets import QWidget

import intermake
import mhelper
import neocommand

from bio42_gui.forms.designer import frm_transfer_designer
from bio42_gui.forms.frm_b42_base import AbstractB42SubWindow


class FrmTransfer( AbstractB42SubWindow ):
    """
    Friendly GUI for the `transfer` command.
    """
    
    
    def __init__( self,
                  parent: QWidget,
                  source: neocommand.Endpoint = None ):
        """
        CONSTRUCTOR
        """
        super().__init__( parent )
        self.ui = frm_transfer_designer.Ui_Dialog( self )
        self.setWindowTitle( "Transfer" )
        
        self.protocols = ComboBoxWrapper( self.ui.CBM_PROTO, cast( Iterable, neocommand.EParcelMethod ), lambda x: x.name )
        self.destinations = ComboBoxWrapper( self.ui.CMB_DEST, (x for x in neocommand.get_core().endpoint_manager if isinstance( x, neocommand.Destination )), lambda x: x.name )
        self.sources = ComboBoxWrapper( self.ui.CMB_SOURCE, (x for x in neocommand.get_core().endpoint_manager if isinstance( x, neocommand.Origin )), lambda x: x.name )
        
        if source is not None:
            if source in self.sources:
                self.sources.selected = source
            else:
                self.sources.append( source, True )
        
        self.ui.CMB_DEST.currentIndexChanged.connect( self.update_info )
        self.ui.CMB_SOURCE.currentIndexChanged.connect( self.update_info )
        self.ui.CBM_PROTO.currentIndexChanged.connect( self.update_info )
        
        self.update_info()
        self.result = None
    
    
    def get_selected( self ):
        return self.sources.selected, self.destinations.selected, self.protocols.selected
    
    
    def get_doc( self, x: neocommand.Endpoint ):
        if x is None:
            return "No endpoint selected"
        else:
            return str( x ) + "\n\n" + mhelper.documentation_helper.get_basic_documentation( x )
    
    
    def update_info( self ):
        s, d, p = self.get_selected()
        
        if p is None:
            tp = "No protocol selected"
        else:
            tp = mhelper.documentation_helper.get_enum_documentation( p )
        
        self.ui.LBL_SRC_INFO.setText( self.get_doc( s ) )
        self.ui.LBL_DEST_INFO.setText( self.get_doc( d ) )
        self.ui.LBL_PROTO_INFO.setText( tp )
    
    
    @exqtSlot()
    def on_BTNBOX_MAIN_accepted( self ) -> None:
        """
        Signal handler:
        """
        s, d, p = self.get_selected()
        
        asr = intermake.Controller.ACTIVE \
            .acquire( neocommand.transfer,
                      window = self ) \
            .run( s, d, p )
        
        asr.listen( self.on_completed )
    
    
    def on_completed( self, result: intermake.Result ):
        if result.is_success:
            self.close()
    
    
    @exqtSlot()
    def on_BTNBOX_MAIN_rejected( self ) -> None:
        """
        Signal handler:
        """
        self.reject()



from typing import *

import os.path
import intermake
import neocommand

from bio42.application import app
from bio42.commands.downloads import _common


@app.command( folder = "import" )
def download_ontology( destination: Optional[neocommand.Destination], refresh: bool = False, clean: bool = False, url: str = "" ) -> None:
    """
    Downloads and optionally imports the Gene Ontology terms.
    
    :param refresh:     Downloads the database database file, even if it already exists on disk.
    :param destination:    Endpoint to import into.
                        `None` = Just download, don't import 
    :param clean:       Delete the downloaded file(s) after importation.
    :param url:         Specifies the location of the database dump to use.
                        If this is left blank, a default location is assumed:
                            
                            http://purl.obolibrary.org/obo/go.owl
                            
                        Suggested values include:
                        
                            GO Core: http://purl.obolibrary.org/obo/go.owl
                            GO Plus: http://purl.obolibrary.org/obo/go/extensions/go-plus.owl
                            
                        Please make sure that you trust the source.
    """
    url = url or "http://purl.obolibrary.org/obo/go.owl"
    dir = intermake.Controller.ACTIVE.app.local_data.local_folder( "downloads", "ontology" )
    file_name = os.path.join( dir, "go.owl" )
    
    if refresh and os.path.isfile( file_name ):
        os.remove( file_name )
        print( "The existing database has been deleted:\n* {}".format( file_name ) )
        
    if not os.path.isfile( file_name ):
        _common.download_file( url, file_name )
    else:
        print( "Not downloading anything because the database file already exists:\n* {}".format( file_name ) )
    
    if destination is None:
        return
    
    from bio42.commands.parsers import parser_go
    
    parser_go.import_go( destination, file_name )
    
    if clean:
        with intermake.pr.pr_action( "Deleting downloaded file" ):
            os.remove( file_name )



from mhelper import ArgInspector as arg
from bio42 import Application
from neocommand import \
    isEdgeLabel as Edge, \
    isNodeLabel as Node, \
    isNodeUid as Uid, \
    ScriptCommand as script, \
    isDbParam as db_param, \
    isScriptParam as py_param


register = Application.INSTANCE.command( folder = "Bio42 Scripts" )

__DISPLAY_LIMIT = 10
__LIST_LIMIT = 15

register( script( name = "Get edge labels",
                  description = "Lists the relationship types in the database",
                  cypher = "MATCH ()-[r]->() RETURN DISTINCT TYPE(r)" ) )

register( script( name = "Count all nodes",
                  description = "Counts the number of nodes in the database",
                  cypher = "MATCH (n) RETURN COUNT(n)" ) )

register( script( name = "Count all edges",
                  description = "Counts the number of edges in the database",
                  cypher = "MATCH ()-[r]->() RETURN COUNT(r)" ) )

register( script( name = "Count nodes with a specified label",
                  description = "Counts the number of nodes with a specific label",
                  cypher = "MATCH (n:<LABEL>) RETURN COUNT(n)",
                  args = [arg( "label", py_param[Node] )] ) )

register( script( name = "Count edges of type",
                  description = "Counts the number of edges with a specific label",
                  cypher = "MATCH (:<START>)-[r:<EDGE>]->(:<START>) RETURN COUNT(r)",
                  args = [arg( "start", py_param[Edge] ),
                          arg( "edge", py_param[Edge] ),
                          arg( "end", py_param[Edge] )] ) )

register( script( name = "Find non-transitive triplets",
                  description = "Finds non-transitive triplets: A LIKE B LIKE C but NOT A LIKE C",
                  cypher = """MATCH p = (n1:Sequence)-[r1:Blast]-(n2:Sequence)-[r2:Blast]-(n3:Sequence)
                              where (not (n1:Sequence)-[:Blast]-(n3:Sequence))
                              return p limit <LIMIT>""",
                  args = [arg( "limit", py_param[int], __DISPLAY_LIMIT )] ) )

register( script( name = "Find non-transitive chains of 5",
                  description = "Finds non-transitive chains of 5..",
                  cypher = """MATCH p = (n1:Sequence)-[r1:Like]-(n2:Sequence)-[r2:Like]-(n3:Sequence)-[r3:Like]-(n4:Sequence)-[r4:Like]-(n5:Sequence)
                      where (not (n1:Sequence)-[:Like]-(n3:Sequence))
                      and (not (n1:Sequence)-[:Like]-(n4:Sequence))
                      and (not (n1:Sequence)-[:Like]-(n5:Sequence))
                      and (not (n2:Sequence)-[:Like]-(n4:Sequence))
                      and (not (n2:Sequence)-[:Like]-(n5:Sequence))
                      and (not (n3:Sequence)-[:Like]-(n5:Sequence))
                      return p limit 1
                      """ ) )

register( script( name = "Find links between organisms",
                  description = "Given two organisms, for which scientific names should be specified, finds the sequences they share in common",
                  cypher = """MATCH (a:Taxon {scientific_name:{first}}), (b:Taxon {scientific_name:{second}})
                  MATCH path = (a)-->(:Sequence)-[:Like]-(:Sequence)<--(b)
                  RETURN path""",
                  args = [arg( "first", db_param[str] ),
                          arg( "second", db_param[str] )] ) )

register( script( name = "Find organism links between sequences",
                  description = """Given a starting sequence, who's UID should be specified, this query lists the names of the species
                       in which a similar sequence occurs.""",
                  cypher = """MATCH (n:Sequence {uid:{uid}})
                  MATCH (n)-[:Like]-(:Sequence)<-[:Contains]-(o:Taxon)
                  WITH DISTINCT o
                  RETURN DISTINCT o.scientific_name
                  """,
                  args = [arg( "uid", db_param[Uid] )] ) )

register( script( name = "List taxa with sequence data",
                  description = "Returns the names of taxa for which sequence data is present",
                  cypher = """MATCH (taxon:Taxon)
                  WHERE (taxon)-[:Contains]->(:Sequence)
                  RETURN taxon.scientific_name
                  """ ) )

register( script( name = "Find composite genes",
                  description = "A quick and dirty method to find composite genes",
                  cypher = """MATCH (a:Sequence)-[ab:Blast]-(b:Sequence)
                  MATCH (a)-[ac:Blast]-(c:Sequence)
                  WHERE a <> b
                  AND b <> c
                  AND ab.end < ac.start
                  RETURN a, ab, b, ac, c
                  LIMIT <LIMIT>
                  """,
                  args = [arg( "limit", py_param[int], __LIST_LIMIT )] ) )

register( script( name = "Show taxon hierarchy",
                  description = "Given a query taxon, shows how it inherits from the root.",
                  cypher = """MATCH p = (a:Taxon [ uid:[uid] ])<-[:Contains*]-(:Taxon [ uid:"1" ])
                  return p
                  """,
                  args = [arg( "uid", py_param[Uid] )] ) )

register( script( name = "Show taxon MRCA path",
                  description = "Given two taxa, [a] and [b], shows how they are related.",
                  cypher = """MATCH p = shortestpath( (a:Taxon [ uid:[a] ])-[:Contains*]-(b:Taxon [ uid:[b] ]) )
                  return p
                  """,
                  args = [arg( "a", db_param[Uid] ),
                          arg( "b", db_param[Uid] )] ) )



from PyQt5.QtWidgets import QWidget, QTreeWidgetItem
from mhelper_qt import exceptToGui, exqtSlot, menu_helper, tree_helper
from typing import Optional

import intermake
import neocommand

from bio42_gui.forms.designer.resources import resources
from bio42_gui.forms.designer import frm_endpoint_list_designer
from bio42_gui.forms.frm_b42_base import AbstractB42SubWindow


class FrmEndpointList( AbstractB42SubWindow ):
    """
    Displays the list of endpoints.
    """
    TAG_ENDPOINT = "_FrmEndpointList__tag_endpoint"
    
    
    @exceptToGui()
    def __init__( self, parent: QWidget ):
        """
        CONSTRUCTOR
        """
        super().__init__( parent )
        self.ui = frm_endpoint_list_designer.Ui_Dialog( self )
        self.setWindowTitle( "Endpoint list" )
        self.update_view()
        tree_helper.set_as_list( self.ui.TVW_MAIN )
    
    
    def update_view( self ):
        self.ui.TVW_MAIN.clear()
        
        headers = QTreeWidgetItem()
        headers.setText( 0, "Name" )
        headers.setText( 1, "Value" )
        self.ui.TVW_MAIN.setHeaderItem( headers )
        
        for endpoint in neocommand.get_core().endpoint_manager:
            endpoint: neocommand.Endpoint = endpoint
            item = QTreeWidgetItem()
            item.setText( 0, endpoint.name )
            item.setText( 1, str( endpoint ) )
            item.setIcon( 0, resources.endpoint.icon() )
            setattr( item, self.TAG_ENDPOINT, endpoint )
            self.ui.TVW_MAIN.addTopLevelItem( item )
        
        tree_helper.resize_all( self.ui.TVW_MAIN )
    
    
    def __command_complete( self, _: intermake.Result ) -> None:
        self.update_view()
    
    
    def get_selected( self ) -> Optional[neocommand.Endpoint]:
        items = self.ui.TVW_MAIN.selectedItems()
        
        if len( items ) != 1:
            return None
        
        item = items[0]
        return getattr( item, self.TAG_ENDPOINT )
    
    
    def has_selected( self ) -> bool:
        return self.get_selected() is not None
    
    
    @exqtSlot()
    def on_BTN_ADD_clicked( self ) -> None:
        """
        Signal handler:
        """
        opts = ["Database", "Parcel", "File"]
        
        m = menu_helper.show_menu( self, opts )
        
        if m == "Database":
            self.show_command( neocommand.endpoints.open_database ).listen( self.__command_complete )
        elif m == "File":
            self.show_command( neocommand.endpoints.open_file ).listen( self.__command_complete )
        elif m == "Parcel":
            self.show_command( neocommand.endpoints.open_parcel ).listen( self.__command_complete )
    
    
    @exqtSlot()
    def on_BTN_REMOVE_clicked( self ) -> None:
        """
        Signal handler:
        """
        if self.has_selected():
            self.show_command( neocommand.endpoints.close, endpoint = self.get_selected(), delete = False ).listen( self.__command_complete )
    
    
    @exqtSlot()
    def on_BTN_DELETE_clicked( self ) -> None:
        """
        Signal handler:
        """
        if self.has_selected():
            self.show_command( neocommand.endpoints.close, endpoint = self.get_selected(), delete = True ).listen( self.__command_complete )
    
    
    @exqtSlot()
    def on_BTN_REFRESH_clicked( self ) -> None:
        """
        Signal handler:
        """
        self.update_view()
    
    
    @exqtSlot()
    def on_BTN_OK_clicked( self ) -> None:
        """
        Signal handler:
        """
        self.close()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/martinrusilowicz/work/apps/bio42/bio42_gui/forms/designer/frm_endpoint_list_designer.ui'
#
# Created by: PyQt5 UI code generator 5.9
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Dialog(object):
    def __init__(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(976, 690)
        self.horizontalLayout = QtWidgets.QHBoxLayout(Dialog)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.TVW_MAIN = QtWidgets.QTreeWidget(Dialog)
        self.TVW_MAIN.setObjectName("TVW_MAIN")
        self.TVW_MAIN.headerItem().setText(0, "1")
        self.horizontalLayout.addWidget(self.TVW_MAIN)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.BTN_ADD = QtWidgets.QCommandLinkButton(Dialog)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/bio42/add.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_ADD.setIcon(icon)
        self.BTN_ADD.setIconSize(QtCore.QSize(32, 32))
        self.BTN_ADD.setObjectName("BTN_ADD")
        self.verticalLayout.addWidget(self.BTN_ADD)
        self.BTN_REMOVE = QtWidgets.QCommandLinkButton(Dialog)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/bio42/disconnect.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_REMOVE.setIcon(icon1)
        self.BTN_REMOVE.setIconSize(QtCore.QSize(32, 32))
        self.BTN_REMOVE.setObjectName("BTN_REMOVE")
        self.verticalLayout.addWidget(self.BTN_REMOVE)
        self.BTN_DELETE = QtWidgets.QCommandLinkButton(Dialog)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/bio42/delete.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_DELETE.setIcon(icon2)
        self.BTN_DELETE.setIconSize(QtCore.QSize(32, 32))
        self.BTN_DELETE.setObjectName("BTN_DELETE")
        self.verticalLayout.addWidget(self.BTN_DELETE)
        self.BTN_REFRESH = QtWidgets.QCommandLinkButton(Dialog)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/bio42/refresh.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_REFRESH.setIcon(icon3)
        self.BTN_REFRESH.setIconSize(QtCore.QSize(32, 32))
        self.BTN_REFRESH.setObjectName("BTN_REFRESH")
        self.verticalLayout.addWidget(self.BTN_REFRESH)
        self.BTN_OK = QtWidgets.QCommandLinkButton(Dialog)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/bio42/verify.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_OK.setIcon(icon4)
        self.BTN_OK.setIconSize(QtCore.QSize(32, 32))
        self.BTN_OK.setObjectName("BTN_OK")
        self.verticalLayout.addWidget(self.BTN_OK)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.horizontalLayout.addLayout(self.verticalLayout)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Dialog"))
        self.BTN_ADD.setText(_translate("Dialog", "Add"))
        self.BTN_ADD.setDescription(_translate("Dialog", "Add a new endpoint"))
        self.BTN_REMOVE.setText(_translate("Dialog", "Remove"))
        self.BTN_REMOVE.setDescription(_translate("Dialog", "Remove the selected endpoint (but not from disk)"))
        self.BTN_DELETE.setText(_translate("Dialog", "Delete"))
        self.BTN_DELETE.setDescription(_translate("Dialog", "Delete the selected endpoint\'s contents from disk"))
        self.BTN_REFRESH.setText(_translate("Dialog", "Refresh"))
        self.BTN_REFRESH.setDescription(_translate("Dialog", "Refresh the information"))
        self.BTN_OK.setText(_translate("Dialog", "OK"))
        self.BTN_OK.setDescription(_translate("Dialog", "Close this window"))





"""
Imports taxa

Normally the taxa are downloaded as a set of DMP files, these will need to be renamed so we can tell the difference between them
Specifically we want NODES.DMP and NAMES.DMP
"""
import os.path
import intermake
import neocommand
from typing import Dict, List
from mhelper import file_helper, isFilename, EFileMode, isOptional

from bio42 import schema
from bio42.application import app

UID_TAXON_UNKNOWN = "unknown"
_EXT_DMP = ".dmp"


@app.command( folder = "import" )
def import_taxonomy( endpoint: neocommand.Destination,
                     file_name: isOptional[isFilename[EFileMode.READ, _EXT_DMP]],
                     names_file_name: isOptional[isFilename[EFileMode.READ, _EXT_DMP]] ) -> None:
    """
    Imports the taxonomy database.
    
    :param endpoint:            Where to send the parsed data to. 
    :param file_name:           Where to get the data from.
                                This should be an NCBI taxonomy database, e.g. `taxa.dmp` 
    :param names_file_name:     Names file.
                                This should be an NCBI taxonomy database, e.g. `names.dmp`
                                If left blank this parameter assumes a file called `names.dmp` in the same directory as `file_name`. 
    :return: 
    """
    #
    # Try and find the names file
    #
    if not names_file_name:
        tax_file_names = [file_helper.replace_filename( file_name, "names.dmp" ), file_helper.replace_extension( file_name, ".names" )]
        for possibility in tax_file_names:
            if os.path.isfile( possibility ):
                names_file_name = possibility
                break
    
    print( "Reading taxonomy from «{}» and «{}».".format( file_name, names_file_name ) )
    
    tax_names = _read_names( names_file_name )
    
    # Always create an unknown
    with endpoint.open_writer() as writer:
        _create_unknown_taxon_node( writer )
        
        with intermake.pr.pr_action( "Iterating nodes" ) as action:
            with open( file_name, "r" ) as file:
                for line in file:
                    action.increment()
                    __yield_line( writer, line, tax_names )


def __yield_line( writer: neocommand.Writer, line: str, tax_names: Dict[int, Dict[str, List[str]]] ) -> None:
    # 00 tax_id                            -- node id in GenBank taxonomy database
    # 01 parent tax_id                     -- parent node id in GenBank taxonomy database
    # 02 rank                              -- rank of this node (superkingdom, kingdom, ...)
    # 03 embl code#                        -- locus-name prefix; not unique
    # 04 division id                       -- see division.dmp file
    # 05 inherited div flag  (1 or 0)      -- 1 if node inherits division from parent
    # 06 genetic code id                   -- see gencode.dmp file
    # 07 inherited GC  flag  (1 or 0)      -- 1 if node inherits genetic code from parent
    # 08 mitochondrial genetic code id     -- see gencode.dmp file
    # 09 inherited MGC flag  (1 or 0)      -- 1 if node inherits mitochondrial gencode from parent
    # 10 GenBank hidden flag (1 or 0)      -- 1 if  name is suppressed in GenBank entry lineage
    # 11 hidden subtree root flag (1 or 0) -- 1 if this subtree has no sequence data yet
    # 12 comments                          -- free-text comments and citations
    
    fields = [x.strip( " \t|\n" ) for x in line.split( "\t|\t" )]
    
    if len( fields ) != 13:
        raise ValueError( "This is not a taxonomy node database." )
    
    # super( ).__init__( fields[ 0 ] )
    
    name = fields[0]
    my_id = int( fields[0] )
    applicable_names = tax_names[my_id]
    
    # Each value of applicable_names is
    # 00 id
    # 01 name
    # 02 unique name
    # 03 class
    if "scientific_name" in applicable_names:
        scientific_name = applicable_names["scientific_name"][0]
    else:
        scientific_name = None
    
    parent_id = int( fields[1] )
    
    props = {
        "parent_id"                    : parent_id,
        "rank"                         : fields[2],  # don't short-code this any more since its reasonably useful
        "embl_code"                    : fields[3],  # letters
        "division_id"                  : int( fields[4] ),
        "inherited_div_flag"           : _str_to_bool( fields[5] ),
        "genetic_code_id"              : int( fields[6] ),
        "inherited_gc_flag"            : _str_to_bool( fields[7] ),
        "mitochondrial_genetic_code_id": int( fields[8] ),
        "inherited_mgc_flag"           : _str_to_bool( fields[9] ),
        "genbank_hidden_flag"          : _str_to_bool( fields[10] ),
        "hidden_subtree_root_flag"     : _str_to_bool( fields[11] ),
        "comments"                     : fields[12],
        "acronym"                      : read_name( applicable_names, "acronym" ),
        "anamorph"                     : read_name( applicable_names, "anamorph" ),
        "authority"                    : read_name( applicable_names, "authority" ),
        "blast_name"                   : read_name( applicable_names, "blast_name" ),
        "common_name"                  : read_name( applicable_names, "common_name" ),
        "equivalent_name"              : read_name( applicable_names, "equivalent_name" ),
        "genbank_acronym"              : read_name( applicable_names, "genbank_acronym" ),
        "genbank_anamorph"             : read_name( applicable_names, "genbank_anamorph" ),
        "genbank_common_name"          : read_name( applicable_names, "genbank_common_name" ),
        "genbank_synonym"              : read_name( applicable_names, "genbank_synonym" ),
        "in_part"                      : read_name( applicable_names, "in_part" ),
        "includes"                     : read_name( applicable_names, "includes" ),
        "misnomer"                     : read_name( applicable_names, "misnomer" ),
        "misspelling"                  : read_name( applicable_names, "misspelling" ),
        "scientific_name"              : scientific_name,
        "scientific_names"             : read_name( applicable_names, "scientific_name" ),
        "synonym"                      : read_name( applicable_names, "synonym" ),
        "teleomorph"                   : read_name( applicable_names, "teleomorph" ),
        "type_material"                : read_name( applicable_names, "type_material" ) }
    
    writer.create_node( label = schema.Taxon,
                        uid = name,
                        properties = props )
    
    writer.create_edge( label = schema.TaxonContainsTaxon,
                        start_uid = str( parent_id ),
                        end_uid = name, properties = { } )


def read_name( dictionary, type_ ):
    if type_ not in dictionary:
        return ""
    
    names = dictionary[type_]
    
    return ", ".join( names )


def _create_unknown_taxon_node( writer: neocommand.Writer ) -> None:
    props = {
        "parent_id"                    : -1,
        "rank"                         : "",
        "embl_code"                    : "",
        "division_id"                  : -1,
        "inherited_div_flag"           : False,
        "genetic_code_id"              : -1,
        "inherited_gc_flag"            : False,
        "mitochondrial_genetic_code_id": -1,
        "inherited_mgc_flag"           : False,
        "genbank_hidden_flag"          : False,
        "hidden_subtree_root_flag"     : False,
        "comments"                     : "Unknown signifier automatically generated by " + intermake.Controller.ACTIVE.app.name + " " + intermake.Controller.ACTIVE.app.version,
        "acronym"                      : "",
        "anamorph"                     : "",
        "authority"                    : "",
        "blast_name"                   : "",
        "common_name"                  : "unknown",
        "equivalent_name"              : "",
        "genbank_acronym"              : "",
        "genbank_anamorph"             : "",
        "genbank_common_name"          : "",
        "genbank_synonym"              : "",
        "in_part"                      : "",
        "includes"                     : "",
        "misnomer"                     : "",
        "misspelling"                  : "",
        "scientific_name"              : "unknown",
        "synonym"                      : "",
        "teleomorph"                   : "",
        "type_material"                : "" }
    
    writer.create_node( label = schema.Taxon,
                        uid = UID_TAXON_UNKNOWN,
                        properties = props )


def _read_names( file_name ) -> Dict[str, Dict[int, List[str]]]:
    """
    Reads a taxonomy names file.
    :param file_name: File to read 
    :return: Dictionary:
                K: Taxon ID
                V: Dictionary:
                    K: Name type (e.g. "scientific_name")
                    V: LIST:
                        V: Name
    """
    assert file_name, "A taxonomy names file must be specified."
    
    result_names: Dict[str, Dict[int, List[str]]] = { }
    
    with intermake.pr.pr_action( "Reading names" ) as action:
        with open( file_name, "r" ) as file:
            for line in file:
                action.increment()
                fields = [x.strip( " \t|\n" ) for x in line.split( "\t|\t" )]
                id = int( fields[0] )
                
                name = fields[1]
                name_type = fields[3].replace( " ", "_" ).replace( "-", "_" )
                
                if id in result_names:
                    dict_for_id = result_names[id]
                else:
                    dict_for_id = { }
                    result_names[id] = dict_for_id
                
                if name_type in dict_for_id:
                    names_for_class = dict_for_id[name_type]
                else:
                    names_for_class = []
                    dict_for_id[name_type] = names_for_class
                
                names_for_class.append( name )
    
    return result_names


def _read_scientific_names( file_name ) -> Dict[int, str]:
    names = _read_names( file_name )
    result: Dict[int, str] = { }
    
    for k, v in names.items():
        scientific_names = v.get( "scientific_name" )
        
        if scientific_names:
            scientific_name = scientific_names[0]
            result[k] = scientific_name
    
    return result


def _str_to_bool( text: str ) -> bool:
    """
    Converts the text to a boolean value.
    """
    if text == "0":
        return False
    elif text == "1":
        return True
    else:
        raise ValueError( "Cannot convert text \"" + text + "\" to bool." )



from PyQt5.QtWidgets import QWidget
from mhelper_qt import exqtSlot

from bio42_gui.forms.designer import frm_getting_started_designer as _my_form
from bio42_gui.forms.frm_b42_base import AbstractB42SubWindow


class FrmGettingStarted( AbstractB42SubWindow ):
    """
    Provides shortcuts to some basic commands to get the user started.
    
    The result of `request`ing this form is one of the `O_*` members, or `None`.
    """
    O_CREATE_PARCEL = object()
    O_PUT_DATA_IN = object()
    O_OPEN_CONNECTION = object()
    O_CREATE_DATABASE = object()
    O_EXPLORE_DATA = object()
    
    
    def __init__( self, parent: QWidget ):
        """
        CONSTRUCTOR
        """
        super().__init__( parent )
        self.ui = _my_form.Ui_Dialog( self )
        self.setWindowTitle( "Getting Started" )
        self.result = None
    
    
    def on_get_result( self ):
        return self.result
    
    
    @exqtSlot()
    def on_BTN_CREATE_PARCEL_clicked( self ) -> None:
        """
        Signal handler:
        """
        self.result = self.O_CREATE_PARCEL
        self.accept()
    
    
    @exqtSlot()
    def on_BTN_PUT_DATA_IN_clicked( self ) -> None:
        """
        Signal handler:
        """
        self.result = self.O_PUT_DATA_IN
        self.accept()
    
    
    @exqtSlot()
    def on_BTN_OPEN_CONNECTION_clicked( self ) -> None:
        """
        Signal handler:
        """
        self.result = self.O_OPEN_CONNECTION
        self.accept()
    
    
    @exqtSlot()
    def on_BTN_CREATE_DATABASE_clicked( self ) -> None:
        """
        Signal handler:
        """
        self.result = self.O_CREATE_DATABASE
        self.accept()
    
    
    @exqtSlot()
    def on_BTN_EXPLORE_DATA_clicked( self ) -> None:
        """
        Signal handler:
        """
        self.result = self.O_EXPLORE_DATA
        self.accept()



import intermake
from mhelper import EFileMode, isFilename, array_helper
import neocommand
import csv
from typing import Dict, List, Optional, Union

from bio42.application import app

TReadCsvFilename = isFilename[EFileMode.READ, ".csv"]
TColId = Union[str, int]


class _HMap:
    def __init__( self, index, text ):
        """
        :param index:   Header index 
        :param text:    Text of the form `'entity.header'` OR `''` to ignore the column. 
        """
        self.index: int = index
        
        if text:
            a, b = text.split( ".", 1 )
            self.entity_str = a
            self.header: neocommand.NcvHeader = neocommand.NcvHeader.from_decorated_name( None, b )
        else:
            self.entity_str = None
            self.header: neocommand.NcvHeader = None
        
        self.entity: _HEnt = None


class _HEnt:
    def __init__( self, label ):
        self.label = label
        self.is_edge: bool = None
        self.uid_col: _HMap = None
        self.start_col: _HMap = None
        self.end_col: _HMap = None
        self.start_label = None
        self.end_label = None
        self.schema = None
        
        self.cur_props: Dict[str, object] = { }
        self.cur_uid: str = None
        self.cur_start: str = None
        self.cur_end: str = None


@app.command( folder = "import" )
def import_csv( file_name: TReadCsvFilename,
                destination: neocommand.Destination,
                headers: bool = True,
                fields: Optional[List[str]] = None,
                mapping: Optional[Dict[Union[int, str], str]] = None,
                delimiter = ",",
                array_delimiter = "¶",
                preview: bool = False ):
    """
    Imports specified data from a CSV file.
        
    :param preview:                 Previews the import headers but doesn't do anything.
    :param file_name:               File to import. 
    :param destination:             Where to send the data to.
    :param headers:                 File has headers?
                                    Headers should be of the form:
                                        `label.property`
                                    Where `property` is a valid Neo4j identifier such as `spam:int` `uid:ID` (see 
                                    `from_decorated_name`).
                                    Empty headers are permitted and indicate that that column is ignored.
                                    Whether `label` denotes a node or edge label is inferred from the properties.
    :param fields:                  Overrides the file headers.
                                    Mandatory if the file has no headers OR if the headers are not in the correct format. 
    :param mapping:                     Alternative to `fields` that specifies headers as key-value pairs.
                                    `key` = the name of the old header, or `#` followed by the zero based index of the header.
                                    `value` = the new header 
    :param delimiter:               CSV delimiter 
    :param array_delimiter:         Array delimiter 
    :return:                        The `endpoint` 
    """
    with open( file_name, "r" ) as i:
        csv_reader = csv.reader( i, delimiter = delimiter )
        maps_in: List[_HMap] = []
        
        # Read the headers, if available
        if headers:
            header_row: List[str] = [cell.strip() for cell in next( csv_reader )]
        else:
            header_row = None
        
        # Convert `fields` to `map`
        if fields is not None:
            mapping = { i: v for i, v in enumerate( fields ) }
        
        # Convert `map` or `header_row` to an `_HMap` array.
        # The `_HMap` constructor parses the actual text.
        if mapping is not None:
            for key, value in mapping.items():
                if isinstance( key, int ):
                    index = key
                elif key.startswith( "#" ):
                    index = int( key[1:] )
                elif header_row is not None:
                    index = header_row.index( key )
                else:
                    raise ValueError( "Cannot specify headers by name when `headers` is `False`." )
                
                maps_in.append( _HMap( index, value ) )
        elif header_row is not None:
            for index, header in enumerate( header_row ):
                maps_in.append( _HMap( index, header ) )
        else:
            raise ValueError( "Must provide `headers`, `fields` or `map`." )
        
        # Sort our `_HMap` array by index and make a note of the entities
        maps: List[_HMap] = []
        entities: Dict[str, _HEnt] = { }
        
        for mapping in maps_in:
            # Discard empty mappings
            if mapping.entity_str is None:
                continue
            
            array_helper.ensure_capacity( maps, index = mapping.index )
            maps[mapping.index] = mapping
            
            ent = entities.get( mapping.entity_str )
            
            if ent is None:
                ent = _HEnt( mapping.entity_str )
                entities[mapping.entity_str] = ent
            
            mapping.entity = ent
            
            if mapping.header.special == neocommand.ENcvSpecial.UID:
                ent.is_edge = False
                ent.uid_col = mapping
            
            if mapping.header.special == neocommand.ENcvSpecial.START:
                ent.is_edge = True
                ent.start_col = mapping
                ent.start_label = mapping.header.type_label
            
            if mapping.header.special == neocommand.ENcvSpecial.END:
                ent.is_edge = True
                ent.end_col = mapping
                ent.end_label = mapping.header.type_label
        
        for ent in entities.values():
            if ent.is_edge:
                assert ent.start_label
                assert ent.label
                assert ent.end_label
                ent.schema = neocommand.get_core().schema.edge_schema.create( ent.start_label, ent.label, ent.end_label )
            else:
                assert not ent.start_label
                assert ent.label
                assert not ent.end_label
                ent.schema = neocommand.get_core().schema.node_schema.create( ent.label )
        
        if preview:
            for index, mapping in enumerate( maps ):
                if mapping is None:
                    print( "COLUMN {} = -".format( index ) )
                else:
                    print( "COLUMN {} = {}".format( index, mapping.entity, mapping.header ) )
            
            for ent in entities:
                if ent.is_edge:
                    print( "EDGE: {}--{}->{} ({}->{})".format( ent.start_col.header.type_label, ent.label, ent.end_col.header.type_label, ent.start_col.index, ent.end_col.index ) )
                else:
                    print( "NODE: {} ({})".format( ent.label, ent.uid_col ) )
            
            return destination
        
        # Iterate our rows
        with destination.open_writer() as dst:
            for row in intermake.pr.pr_iterate( csv_reader, "Iterating rows" ):
                for ent in entities.values():
                    ent.cur_props.clear()
                    ent.cur_start = None
                    ent.cur_end = None
                    ent.cur_uid = None
                
                for index, cell in enumerate( row ):
                    cell = cell.strip()
                    mapping: _HMap = maps[index]
                    
                    if mapping is None:
                        continue
                    
                    ent: _HEnt = mapping.entity
                    
                    if mapping.header.special == neocommand.ENcvSpecial.UID:
                        ent.cur_uid = cell
                    elif mapping.header.special == neocommand.ENcvSpecial.START:
                        ent.cur_start = cell
                    elif mapping.header.special == neocommand.ENcvSpecial.END:
                        ent.cur_end = cell
                    elif mapping.header.type.is_array:
                        ent.cur_props[mapping.header.name] = (mapping.header.type.element_type( x ) for x in cell.split( array_delimiter ))
                    else:
                        ent.cur_props[mapping.header.name] = mapping.header.type.element_type( cell )
                
                for ent in entities.values():
                    if ent.is_edge:
                        dst.create_edge( label = ent.schema,
                                         start_uid = ent.cur_start,
                                         end_uid = ent.cur_end,
                                         properties = ent.cur_props )
                    else:
                        dst.create_node( label = ent.schema,
                                         uid = ent.cur_uid,
                                         properties = ent.cur_props )
    
    return destination


def __read_properties( row: List[str], indices: List[int], names: Optional[List[str]], headers: Optional[List[str]] ):
    r = { }
    
    for order, index in enumerate( indices ):
        if names is not None:
            name = names[order]
        elif headers is not None:
            name = headers[index]
        else:
            raise ValueError( "To get the column name I need either a `name` or `header`, but I have neither." )
        
        r[name] = row[index]
    
    return r



"""Resources names. File automatically generated by PyGuiSignal. Modifications may be overwritten."""
from mhelper import ResourceIcon
add = ResourceIcon( ":/bio42/add.svg" )
annotation = ResourceIcon( ":/bio42/annotation.svg" )
browse = ResourceIcon( ":/bio42/browse.svg" )
browser = ResourceIcon( ":/bio42/browser.svg" )
csv = ResourceIcon( ":/bio42/csv.svg" )
database = ResourceIcon( ":/bio42/database.svg" )
delete = ResourceIcon( ":/bio42/delete.svg" )
disconnect = ResourceIcon( ":/bio42/disconnect.svg" )
endpoint = ResourceIcon( ":/bio42/endpoint.svg" )
file = ResourceIcon( ":/bio42/file.svg" )
go = ResourceIcon( ":/bio42/go.svg" )
go_to_top = ResourceIcon( ":/bio42/go_to_top.svg" )
help = ResourceIcon( ":/bio42/help.svg" )
information = ResourceIcon( ":/bio42/information.svg" )
network = ResourceIcon( ":/bio42/network.svg" )
next = ResourceIcon( ":/bio42/next.svg" )
optimise = ResourceIcon( ":/bio42/optimise.svg" )
package = ResourceIcon( ":/bio42/package.svg" )
parcel = ResourceIcon( ":/bio42/parcel.svg" )
pickle = ResourceIcon( ":/bio42/pickle.svg" )
refresh = ResourceIcon( ":/bio42/refresh.svg" )
run_query = ResourceIcon( ":/bio42/run_query.svg" )
save = ResourceIcon( ":/bio42/save.svg" )
script = ResourceIcon( ":/bio42/script.svg" )
select_database = ResourceIcon( ":/bio42/select_database.svg" )
send = ResourceIcon( ":/bio42/send.svg" )
sequence = ResourceIcon( ":/bio42/sequence.svg" )
settings = ResourceIcon( ":/bio42/settings.svg" )
similarity = ResourceIcon( ":/bio42/similarity.svg" )
start = ResourceIcon( ":/bio42/start.svg" )
stop = ResourceIcon( ":/bio42/stop.svg" )
taxonomy = ResourceIcon( ":/bio42/taxonomy.svg" )
type_edge = ResourceIcon( ":/bio42/type_edge.svg" )
type_endpoint = ResourceIcon( ":/bio42/type_endpoint.svg" )
type_graph = ResourceIcon( ":/bio42/type_graph.svg" )
type_node = ResourceIcon( ":/bio42/type_node.svg" )
verify = ResourceIcon( ":/bio42/verify.svg" )
view_browser = ResourceIcon( ":/bio42/view_browser.svg" )
view_list = ResourceIcon( ":/bio42/view_list.svg" )
webpage = ResourceIcon( ":/bio42/webpage.svg" )
whiteboard = ResourceIcon( ":/bio42/whiteboard.png" )
wizard = ResourceIcon( ":/bio42/wizard.svg" )


from bio42_gui.forms.frm_b42main import FrmB42Main

import intermake
import intermake_qt


class B42GuiSettings:
    """
    :ivar auto_load_bio42_scripts: Load `bio42_scripts` library scripts when showing the query window
    """
    KEY = "b42_gui_settings"
    
    
    def __init__( self ):
        self.auto_load_bio42_scripts = True


class B42GuiController( intermake_qt.GuiControllerWithBrowser ):
    def __init__( self ):
        super().__init__()
        self.b42_gui_settings = intermake.Controller.ACTIVE.app.local_data.bind( B42GuiSettings.KEY, B42GuiSettings() )
        
        import bio42_gui.controller.editorium
        bio42_gui.controller.editorium.register_with( self.editorium )
    
    
    def on_create_window( self, args ):
        # Initialise resources
        # noinspection PyUnresolvedReferences
        from bio42_gui.forms.designer.resources import resources_rc
        
        # Get main window
        return FrmB42Main( self )



import intermake


def init( app: intermake.Application ):
    app.help.add( "user_edges", "Specifying user-defined edges",
                  """
                  These are used to create 'X-->CONTAINS-->SEQUENCE' relationships.
                                                            
                  Specify a list of key-value pairs:
                  
                     `property=label,property=label`
                     
                  * `property` is the name of the sequence property used to obtain the UID of the target
                  * `label` is the label (node-type) of the target.
                     
                  Example:
                     
                     `_organism=Taxon,_plasmid=Plasmid`
                  
                  A special case is made when 'Taxon' is specified as the `label`. This allows `property` to specify the 
                  'scientific_name', rather than the 'uid', providing the `taxa` argument is specified.
                  
                  Note that for FASTA files, the properties are specified in the `id_parts` argument, but for GENBANK files they
                  are automatically inferred from the available data. 
                  
                  If you have trouble determining what the property names are, you can trial-import a smaller portion of your file, 
                  sending the data to the ECHO endpoint.
                  """ )
    
    app.help.add( "id_parts", "Specifying accessions",
                  """
                  Many FASTA files have identifiers in a `|` separated format, e.g.::
                      
                     >eggs|beans|spam
                     
                  * The `beans` sequence
                  * The `eggs` database
                  * The `spam` organism.
                  
                  The ``id_parts`` parameter on ``import_sequences`` allows you to pull apart these
                  identifiers. Specify a list of strings denoting the names of the properties for
                  each component, e.g.::
                  
                     sequence|database|organism
                  
                  You should specify as many values as are in your FASTA file.
                  Values may be blank, in which case these elements in the identifier are ignored.
                  You will normally use one of these properties as the `uid` of the node, which is
                  the accession: If not at least one of the elements is ``uid``, the entirety of
                  the name is used as the accession (``eggs|beans|spam`` in the example above).
                  
                  A simple example for a common FASTA file with extraneous information is::
                  
                     ,,,uid,
                     
                  This ignores elements 1, 2, 3 and 5, and assumes the 4th element as the accession. 
                  """ )



import getpass
import os.path
import sys

import mhelper
import neocommand
import bio42


class Config:
    def __init__( self ):
        assert bio42.Application.is_running
        self.sample_root = sys.argv[1] if len( sys.argv > 1 ) else "/Users/martinrusilowicz/work/data/plasmids/genbank"
        table = [mhelper.file_helper.list_dir( self.sample_root )]
        self.genbank_file = self.__find_file( table, "all.gb", "b1b2ec60136ad9ec9c4f89d212594f8dc230f92b" ),
        self.annotations_file = self.__find_file( table, "seq-to-familiy-and-go.tsv", "ddc8657c1d4a342044b212de0ff2bfe325d57e19" )
        self.blast_file = self.__find_file( table, "todo", "????" )
        self.coincidence_family_file = self.__find_file( table, "seq-to-family.csv", "????" )
        self.coincidence_edge_file = self.__find_file( table, "significant.csv", "????" )
        self.db_driver = "neo4jv1"
        self.db_host = "127.0.0.1"
        self.db_user = "neo4j"
        self.db_directory = os.path.expanduser( "~/bin/neo4j330" )
        self.db_operating_system = neocommand.EOperatingSystem.UNIX
        self.db_port = 7474
        self.coincidence_family_map = { "sequence": "Sequence.uid:ID(Sequence)",
                                        "family"  : "Family.uid:ID(Family)" }
        self.coincidence_edge_map = { ""      : "",
                                      "Source": "",
                                      "Target": "",
                                      "name_i": "Association.:START_ID(Family)",
                                      "name_j": "Association.:END_ID(Family)",
                                      "p"     : "Association.p:float",
                                      "suc"   : "Association.suc:int",
                                      "obs"   : "Association.obs:int",
                                      "rate"  : "Association.rate:float",
                                      "exp"   : "Association.exp:float",
                                      "tot_i" : "Association.tot_i:int",
                                      "tot_j" : "Association.tot_j:int",
                                      "frac_i": "Association.frac_i:int",
                                      "frac_j": "Association.frac_j:int" }
        self.genbank_user_edges = ["_organism=Taxon"] 
        self.refresh_downloads = True
        self.remove_downloads = True
        print( "The following assumptions have been made:" )
        print( "\n".join( "{}: {}".format( k, v ) for k, v in self.__dict__.items() ) )
        self.db_password = getpass.getpass( "Database password: " )
    
    
    def __find_file( self, table, name, hash ):
        poss = []
        
        for x in table:
            if mhelper.file_helper.get_file_name( x ) == name:
                poss.append( x )
        
        if not poss:
            raise ValueError( "The data provided is missing the required file - name: '{}', hash: '{}'. There is no file with this name.".format( name, hash ) )
        elif len( poss ) > 1:
            raise ValueError( "The data provided is missing the required file - name: '{}', hash: '{}'. There are multiple files with this name ({}).".format( name, hash, poss ) )
        
        file_name = poss[0]
        
        achash = mhelper.io_helper.hash_file( file_name )
        
        if achash != hash:
            raise ValueError( "The data provided is missing the required file - name: '{}', hash: '{}'. There is a file with this name ({}) but the hash is different ({}).".format( name, hash, file_name, achash ) )
        
        return file_name


def log( message ):
    print( mhelper.ansi.BACK_BLUE + mhelper.ansi.FORE_WHITE + message + mhelper.ansi.RESET )



import urllib.request
import intermake
import os
from mhelper import string_helper


def download_file( url, zip_fn ):
    BUFFER = 1000
    tmp_fn = zip_fn + ".download.tmp"
    print( "Downloading: {} --> {}".format( url, tmp_fn ) )
    with intermake.pr.pr_action( "Downloading", text = string_helper.format_size ) as action:
        with urllib.request.urlopen( url ) as input:
            with open( tmp_fn, "wb" ) as output:
                while True:
                    x = input.read( BUFFER )
                    
                    if not x:
                        break
                    
                    output.write( x )
                    action.increment( len( x ) )
        
        os.replace( tmp_fn, zip_fn )



from mhelper import isFilename, EFileMode, string_helper
import intermake
import neocommand

from bio42.application import app
from bio42 import schema


_OWL_EXT = ".owl"


@app.command( folder = "import" )
def import_go( endpoint: neocommand.Destination,
               file_name: isFilename[EFileMode.READ, _OWL_EXT] ) -> None:
    """
    Imports GO terms from the GO database.
    
    :param endpoint: Where to send the data to. 
    :param file_name:   Where to get the data from 
    """
    from xml.etree import ElementTree
    from xml.etree.ElementTree import Element
    
    _GO_ID_PREFIX = "GO:"
    
    with intermake.pr.pr_action( "Parsing GO tree" ):
        tree = ElementTree.parse( file_name )
    
    root = tree.getroot()
    
    CLASS_ID = '{http://www.w3.org/2002/07/owl#}Class'
    CLASS_SUBCLASS_ID = '{http://www.w3.org/2000/01/rdf-schema#}subClassOf'
    CLASS_SUBCLASS_RESOURCE_ID = '{http://www.w3.org/1999/02/22-rdf-syntax-ns#}resource'
    CLASS_ID_ID = '{http://www.geneontology.org/formats/oboInOwl#}id'
    GO_SUBCLASS_ID_REMOVE = 'http://purl.obolibrary.org/obo/GO_'
    
    
    def remove_namespace( tag ):
        tag = str( tag )
        
        if "}" in tag:
            return tag[tag.index( "}" ) + 1:]
        
        return tag
    
    
    with endpoint.open_writer() as writer:
        with intermake.pr.pr_action( "Parsing GO nodes" ) as action:
            for element in root:
                if not isinstance( element, Element ):
                    continue
                
                action.increment()
                
                if element.tag == CLASS_ID:
                    class_subclass = []
                    class_misc = { }
                    class_id = None
                    
                    for sub_element in element:
                        if not isinstance( element, Element ):
                            continue
                        
                        if sub_element.tag == CLASS_SUBCLASS_ID:
                            resource_id = sub_element.get( CLASS_SUBCLASS_RESOURCE_ID )
                            if resource_id:
                                class_subclass.append( string_helper.remove_prefix( resource_id, GO_SUBCLASS_ID_REMOVE ) )
                            continue
                        elif sub_element.tag == CLASS_ID_ID:
                            class_id = string_helper.remove_prefix( sub_element.text, _GO_ID_PREFIX )
                            continue
                        else:
                            key = remove_namespace( sub_element.tag )
                        
                        if key == "IAO_0000115":  # because it's ridiculous
                            key = "description"
                        elif key == "hasOBONamespace":  # because it's also quite silly
                            key = "namespace"
                        elif key == "label":  # because having "node.label" and "node.data.label" as two different things is confusing
                            key = "name"
                        elif key.startswith( "has" ):  # because this makes it sound like a boolean, but it isn't, it's the value
                            key = key[len( "has" ):]
                        
                        key = key.lower()
                        
                        if sub_element.text:
                            text = sub_element.text.strip()
                            
                            if text:
                                if key in class_misc:
                                    if not isinstance( class_misc[key], list ):
                                        class_misc[key] = [class_misc[key]]
                                    
                                    class_misc[key].append( text )
                                else:
                                    class_misc[key] = text
                    
                    if class_id:
                        writer.create_node( label = schema.Go,
                                            uid = class_id,
                                            properties = class_misc )
                        
                        for subclass in class_subclass:
                            writer.create_edge( label = schema.GoClassContainsGoClass,
                                                start_uid = subclass,
                                                end_uid = class_id,
                                                properties = { } )



from PyQt5.QtWidgets import QComboBox
from typing import Optional, Sequence, Union
from mhelper import MEnum
import neocommand


class ESchema( MEnum ):
    NODE = 1
    EDGE = 2
    NODE_PROPERTY = 4
    EDGE_PROPERTY = 8


def add_schema( target: QComboBox, mode: ESchema, left: Optional[Sequence[neocommand.EntitySchema]], right: Optional[Union[Sequence[type], Sequence[neocommand.EntitySchema]]] ):
    SchemaComboBox( target, mode, left, right )


class SchemaComboBox:
    def __init__( self, target: QComboBox, mode: ESchema, left: Optional[Sequence[neocommand.EntitySchema]], right: Optional[Union[Sequence[type], Sequence[neocommand.EntitySchema]]] ):
        """
        
        :param target:      Target. 
        :param mode:        Mode of display.
        :param left:        If `NODE`:          Filter: *      - [left]    -> [options]
                               `EDGE`:          Filter: [left] - [options] -> *
                               `NODE_PROPERTY`: Filter: [left].[options] : *
                               `EDGE_PROPERTY`: Filter: [left].[options] : *
        :param right:       If `NODE`:          Filter: [options] - [right]    -> *
                               `EDGE`:          Filter: *         - [options]  -> [right]
                               `NODE_PROPERTY`: Filter: *.[options] : [right]
                               `EDGE_PROPERTY`: Filter: *.[options] : [right]
        """
        self.target: QComboBox = target
        self.mode: ESchema = mode
        self.left: Optional[Sequence[neocommand.EntitySchema]] = left
        self.right: Optional[Union[Sequence[type], Sequence[neocommand.EntitySchema]]] = right
        self.update()
    
    
    def update( self ):
        self.target.clear()
        
        if self.mode == ESchema.NODE:
            for node in neocommand.get_core().schema.node.find( self.left, self.right ):
                self.target.addItem( node.label )
        
        if self.mode == ESchema.EDGE_PROPERTY:
            for node in neocommand.get_core().schema.edges.find( self.left, self.right ):
                self.target.addItem( node.label )
        
        if self.mode in (ESchema.NODE_PROPERTY, ESchema.EDGE_PROPERTY):
            if self.left is not None:
                lst = self.left
            elif self.mode == ESchema.NODE_PROPERTY:
                lst = neocommand.get_core().schema.nodes
            else:
                lst = neocommand.get_core().schema.edges
            
            for entity in lst:
                for prop in entity.properties:
                    if self.right is not None:
                        if not any( prop.type is x for x in self.right ):
                            continue
                    
                    self.target.addItem( prop.name )



"""
This module contains the primary `Bio42`:t: test case, this processes a large
dataset and exemplifies most of the commands within `Bio42`:t: and
`NeoCommand`:t:.

* Before commencing the test ensure Neo4j is installed and working!
* You'll need the `Paris Plasmids` dataset. 

No binary is created to run this test suite, to run it please invoke Python::

    python -m bio42_tests xxx
    
Where xxx is the directory containing the dataset.
    
If you get an `ExistsError` it's because the test has already been run.
You'll need to clean up manually before starting again.
    
The `__main__.py` contains the import script - this is documented, for example
purposes, inline. 
"""



"""
Subpackage containing the commands associated with downloading information.
"""

from .accessions import download_accessions, download_annotations
from .taxonomy import download_taxonomy
from .ontology import download_ontology


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/martinrusilowicz/work/apps/bio42/bio42_gui/forms/designer/frm_database_designer.ui'
#
# Created by: PyQt5 UI code generator 5.9
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Dialog(object):
    def __init__(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(790, 695)
        self.verticalLayout = QtWidgets.QVBoxLayout(Dialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.groupBox = QtWidgets.QGroupBox(Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox.sizePolicy().hasHeightForWidth())
        self.groupBox.setSizePolicy(sizePolicy)
        self.groupBox.setMinimumSize(QtCore.QSize(0, 32))
        self.groupBox.setStyleSheet("")
        self.groupBox.setTitle("")
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_3.setSpacing(8)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.label_5 = QtWidgets.QLabel(self.groupBox)
        self.label_5.setObjectName("label_5")
        self.verticalLayout_3.addWidget(self.label_5)
        self.TXT_SERVER = QtWidgets.QLineEdit(self.groupBox)
        self.TXT_SERVER.setReadOnly(True)
        self.TXT_SERVER.setObjectName("TXT_SERVER")
        self.verticalLayout_3.addWidget(self.TXT_SERVER)
        self.label_2 = QtWidgets.QLabel(self.groupBox)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_3.addWidget(self.label_2)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setSpacing(0)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.LBL_STATUS_COLOUR = QtWidgets.QLabel(self.groupBox)
        self.LBL_STATUS_COLOUR.setMinimumSize(QtCore.QSize(10, 0))
        self.LBL_STATUS_COLOUR.setMaximumSize(QtCore.QSize(10, 16777215))
        self.LBL_STATUS_COLOUR.setText("")
        self.LBL_STATUS_COLOUR.setObjectName("LBL_STATUS_COLOUR")
        self.horizontalLayout_2.addWidget(self.LBL_STATUS_COLOUR)
        self.TXT_STATUS = QtWidgets.QLineEdit(self.groupBox)
        self.TXT_STATUS.setReadOnly(True)
        self.TXT_STATUS.setObjectName("TXT_STATUS")
        self.horizontalLayout_2.addWidget(self.TXT_STATUS)
        self.verticalLayout_3.addLayout(self.horizontalLayout_2)
        self.label_4 = QtWidgets.QLabel(self.groupBox)
        self.label_4.setObjectName("label_4")
        self.verticalLayout_3.addWidget(self.label_4)
        self.TXT_DATABASE = QtWidgets.QLineEdit(self.groupBox)
        self.TXT_DATABASE.setReadOnly(True)
        self.TXT_DATABASE.setObjectName("TXT_DATABASE")
        self.verticalLayout_3.addWidget(self.TXT_DATABASE)
        self.verticalLayout.addWidget(self.groupBox)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.groupBox_2 = QtWidgets.QGroupBox(Dialog)
        self.groupBox_2.setTitle("")
        self.groupBox_2.setObjectName("groupBox_2")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.groupBox_2)
        self.verticalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_4.setSpacing(12)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.BTN_ENDPOINT = QtWidgets.QCommandLinkButton(self.groupBox_2)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/bio42/database.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_ENDPOINT.setIcon(icon)
        self.BTN_ENDPOINT.setObjectName("BTN_ENDPOINT")
        self.verticalLayout_4.addWidget(self.BTN_ENDPOINT)
        self.GRP_ENDPOINT = QtWidgets.QGroupBox(self.groupBox_2)
        self.GRP_ENDPOINT.setTitle("")
        self.GRP_ENDPOINT.setObjectName("GRP_ENDPOINT")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.GRP_ENDPOINT)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.CMB_ENDPOINT = QtWidgets.QComboBox(self.GRP_ENDPOINT)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.CMB_ENDPOINT.sizePolicy().hasHeightForWidth())
        self.CMB_ENDPOINT.setSizePolicy(sizePolicy)
        self.CMB_ENDPOINT.setObjectName("CMB_ENDPOINT")
        self.horizontalLayout.addWidget(self.CMB_ENDPOINT)
        self.BTN_UPDATE_ENDPOINT = QtWidgets.QPushButton(self.GRP_ENDPOINT)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/bio42/refresh.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_UPDATE_ENDPOINT.setIcon(icon1)
        self.BTN_UPDATE_ENDPOINT.setIconSize(QtCore.QSize(16, 16))
        self.BTN_UPDATE_ENDPOINT.setObjectName("BTN_UPDATE_ENDPOINT")
        self.horizontalLayout.addWidget(self.BTN_UPDATE_ENDPOINT)
        self.verticalLayout_4.addWidget(self.GRP_ENDPOINT)
        self.BTN_START = QtWidgets.QCommandLinkButton(self.groupBox_2)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/bio42/start.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_START.setIcon(icon2)
        self.BTN_START.setObjectName("BTN_START")
        self.verticalLayout_4.addWidget(self.BTN_START)
        self.BTN_STOP = QtWidgets.QCommandLinkButton(self.groupBox_2)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/bio42/stop.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_STOP.setIcon(icon3)
        self.BTN_STOP.setObjectName("BTN_STOP")
        self.verticalLayout_4.addWidget(self.BTN_STOP)
        self.BTN_REFRESH = QtWidgets.QCommandLinkButton(self.groupBox_2)
        self.BTN_REFRESH.setIcon(icon1)
        self.BTN_REFRESH.setObjectName("BTN_REFRESH")
        self.verticalLayout_4.addWidget(self.BTN_REFRESH)
        self.BTN_DATABASE = QtWidgets.QCommandLinkButton(self.groupBox_2)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/bio42/select_database.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_DATABASE.setIcon(icon4)
        self.BTN_DATABASE.setObjectName("BTN_DATABASE")
        self.verticalLayout_4.addWidget(self.BTN_DATABASE)
        self.GRP_DATABASE = QtWidgets.QGroupBox(self.groupBox_2)
        self.GRP_DATABASE.setTitle("")
        self.GRP_DATABASE.setObjectName("GRP_DATABASE")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.GRP_DATABASE)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.CMB_DATABASE = QtWidgets.QComboBox(self.GRP_DATABASE)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.CMB_DATABASE.sizePolicy().hasHeightForWidth())
        self.CMB_DATABASE.setSizePolicy(sizePolicy)
        self.CMB_DATABASE.setObjectName("CMB_DATABASE")
        self.horizontalLayout_4.addWidget(self.CMB_DATABASE)
        self.BTN_NEWDB = QtWidgets.QPushButton(self.GRP_DATABASE)
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/bio42/add.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_NEWDB.setIcon(icon5)
        self.BTN_NEWDB.setIconSize(QtCore.QSize(16, 16))
        self.BTN_NEWDB.setObjectName("BTN_NEWDB")
        self.horizontalLayout_4.addWidget(self.BTN_NEWDB)
        self.BTN_APPLYDB = QtWidgets.QPushButton(self.GRP_DATABASE)
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/bio42/save.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_APPLYDB.setIcon(icon6)
        self.BTN_APPLYDB.setIconSize(QtCore.QSize(16, 16))
        self.BTN_APPLYDB.setObjectName("BTN_APPLYDB")
        self.horizontalLayout_4.addWidget(self.BTN_APPLYDB)
        self.verticalLayout_4.addWidget(self.GRP_DATABASE)
        self.BTN_OPEN_BROWSER = QtWidgets.QCommandLinkButton(self.groupBox_2)
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/bio42/browser.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_OPEN_BROWSER.setIcon(icon7)
        self.BTN_OPEN_BROWSER.setObjectName("BTN_OPEN_BROWSER")
        self.verticalLayout_4.addWidget(self.BTN_OPEN_BROWSER)
        self.BTN_SHOW_CONFIG = QtWidgets.QCommandLinkButton(self.groupBox_2)
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(":/bio42/settings.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_SHOW_CONFIG.setIcon(icon8)
        self.BTN_SHOW_CONFIG.setObjectName("BTN_SHOW_CONFIG")
        self.verticalLayout_4.addWidget(self.BTN_SHOW_CONFIG)
        self.verticalLayout.addWidget(self.groupBox_2)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Dialog"))
        self.label_5.setText(_translate("Dialog", "Server"))
        self.label_2.setText(_translate("Dialog", "Status"))
        self.label_4.setText(_translate("Dialog", "Database"))
        self.BTN_ENDPOINT.setText(_translate("Dialog", "Select server"))
        self.BTN_ENDPOINT.setDescription(_translate("Dialog", "Select the Neo4j server you\'d like to control."))
        self.CMB_ENDPOINT.setToolTip(_translate("Dialog", "Endpoint to view or modify"))
        self.BTN_UPDATE_ENDPOINT.setToolTip(_translate("Dialog", "Change database - you can only do this whilst the server is stopped."))
        self.BTN_UPDATE_ENDPOINT.setText(_translate("Dialog", "Update"))
        self.BTN_START.setToolTip(_translate("Dialog", "Start server"))
        self.BTN_START.setText(_translate("Dialog", "Start"))
        self.BTN_START.setDescription(_translate("Dialog", "Start the Neo4j server."))
        self.BTN_STOP.setToolTip(_translate("Dialog", "Stop server"))
        self.BTN_STOP.setText(_translate("Dialog", "Stop"))
        self.BTN_STOP.setDescription(_translate("Dialog", "Stop the Neo4j server."))
        self.BTN_REFRESH.setText(_translate("Dialog", "Refresh information"))
        self.BTN_REFRESH.setDescription(_translate("Dialog", "Update the information shown on this display"))
        self.BTN_DATABASE.setText(_translate("Dialog", "Change database"))
        self.BTN_DATABASE.setDescription(_translate("Dialog", "Select a new database. Your server must be stopped first."))
        self.CMB_DATABASE.setToolTip(_translate("Dialog", "Change database - you can only do this whilst the server is stopped."))
        self.BTN_NEWDB.setToolTip(_translate("Dialog", "Change database - you can only do this whilst the server is stopped."))
        self.BTN_NEWDB.setText(_translate("Dialog", "New"))
        self.BTN_APPLYDB.setToolTip(_translate("Dialog", "Change database - you can only do this whilst the server is stopped."))
        self.BTN_APPLYDB.setText(_translate("Dialog", "Apply"))
        self.BTN_OPEN_BROWSER.setText(_translate("Dialog", "Open in browser"))
        self.BTN_OPEN_BROWSER.setDescription(_translate("Dialog", "Open the Neo4j browser for the selected database."))
        self.BTN_SHOW_CONFIG.setText(_translate("Dialog", "Select configuration file in Explorer"))
        self.BTN_SHOW_CONFIG.setDescription(_translate("Dialog", "Shows the Neo4j configuration file in your system explorer."))





# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/martinrusilowicz/work/apps/bio42/bio42_gui/forms/designer/frm_b42main_designer.ui'
#
# Created by: PyQt5 UI code generator 5.9
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1521, 1246)
        MainWindow.setStyleSheet("")
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setObjectName("gridLayout")
        self.PAGER_MAIN = QtWidgets.QTabWidget(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.PAGER_MAIN.sizePolicy().hasHeightForWidth())
        self.PAGER_MAIN.setSizePolicy(sizePolicy)
        self.PAGER_MAIN.setDocumentMode(True)
        self.PAGER_MAIN.setObjectName("PAGER_MAIN")
        self.PAG_INTRO = QtWidgets.QWidget()
        self.PAG_INTRO.setObjectName("PAG_INTRO")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.PAG_INTRO)
        self.gridLayout_2.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.LBL_INTRO = QtWidgets.QLabel(self.PAG_INTRO)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.LBL_INTRO.sizePolicy().hasHeightForWidth())
        self.LBL_INTRO.setSizePolicy(sizePolicy)
        self.LBL_INTRO.setStyleSheet("background: white;")
        self.LBL_INTRO.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.LBL_INTRO.setWordWrap(True)
        self.LBL_INTRO.setObjectName("LBL_INTRO")
        self.gridLayout_2.addWidget(self.LBL_INTRO, 0, 0, 1, 1)
        self.PAGER_MAIN.addTab(self.PAG_INTRO, "")
        self.PAG_ENDPOINTS = QtWidgets.QWidget()
        self.PAG_ENDPOINTS.setObjectName("PAG_ENDPOINTS")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.PAG_ENDPOINTS)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.frame = QtWidgets.QFrame(self.PAG_ENDPOINTS)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame.sizePolicy().hasHeightForWidth())
        self.frame.setSizePolicy(sizePolicy)
        self.frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame.setProperty("style", "contents")
        self.frame.setObjectName("frame")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.frame)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.label_2 = QtWidgets.QLabel(self.frame)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_2.addWidget(self.label_2)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.BTN_WIZARD = QtWidgets.QToolButton(self.frame)
        self.BTN_WIZARD.setMinimumSize(QtCore.QSize(96, 96))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/bio42/wizard.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_WIZARD.setIcon(icon)
        self.BTN_WIZARD.setIconSize(QtCore.QSize(64, 64))
        self.BTN_WIZARD.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.BTN_WIZARD.setProperty("style", "item")
        self.BTN_WIZARD.setObjectName("BTN_WIZARD")
        self.horizontalLayout_5.addWidget(self.BTN_WIZARD)
        self.BTN_ENDPOINT_DATABASE = QtWidgets.QToolButton(self.frame)
        self.BTN_ENDPOINT_DATABASE.setMinimumSize(QtCore.QSize(96, 96))
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/bio42/database.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_ENDPOINT_DATABASE.setIcon(icon1)
        self.BTN_ENDPOINT_DATABASE.setIconSize(QtCore.QSize(64, 64))
        self.BTN_ENDPOINT_DATABASE.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.BTN_ENDPOINT_DATABASE.setProperty("style", "item")
        self.BTN_ENDPOINT_DATABASE.setObjectName("BTN_ENDPOINT_DATABASE")
        self.horizontalLayout_5.addWidget(self.BTN_ENDPOINT_DATABASE)
        self.BTN_ENDPOINT_PACKAGE = QtWidgets.QToolButton(self.frame)
        self.BTN_ENDPOINT_PACKAGE.setMinimumSize(QtCore.QSize(96, 96))
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/bio42/parcel.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_ENDPOINT_PACKAGE.setIcon(icon2)
        self.BTN_ENDPOINT_PACKAGE.setIconSize(QtCore.QSize(64, 64))
        self.BTN_ENDPOINT_PACKAGE.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.BTN_ENDPOINT_PACKAGE.setProperty("style", "item")
        self.BTN_ENDPOINT_PACKAGE.setObjectName("BTN_ENDPOINT_PACKAGE")
        self.horizontalLayout_5.addWidget(self.BTN_ENDPOINT_PACKAGE)
        self.BTN_ENDPOINT_MGRAPH = QtWidgets.QToolButton(self.frame)
        self.BTN_ENDPOINT_MGRAPH.setMinimumSize(QtCore.QSize(96, 96))
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/bio42/file.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_ENDPOINT_MGRAPH.setIcon(icon3)
        self.BTN_ENDPOINT_MGRAPH.setIconSize(QtCore.QSize(64, 64))
        self.BTN_ENDPOINT_MGRAPH.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.BTN_ENDPOINT_MGRAPH.setProperty("style", "item")
        self.BTN_ENDPOINT_MGRAPH.setObjectName("BTN_ENDPOINT_MGRAPH")
        self.horizontalLayout_5.addWidget(self.BTN_ENDPOINT_MGRAPH)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem)
        self.verticalLayout_2.addLayout(self.horizontalLayout_5)
        self.label_4 = QtWidgets.QLabel(self.frame)
        self.label_4.setObjectName("label_4")
        self.verticalLayout_2.addWidget(self.label_4)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.BTN_ENDPOINT_TRANSFER = QtWidgets.QToolButton(self.frame)
        self.BTN_ENDPOINT_TRANSFER.setMinimumSize(QtCore.QSize(96, 96))
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/bio42/send.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_ENDPOINT_TRANSFER.setIcon(icon4)
        self.BTN_ENDPOINT_TRANSFER.setIconSize(QtCore.QSize(64, 64))
        self.BTN_ENDPOINT_TRANSFER.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.BTN_ENDPOINT_TRANSFER.setProperty("style", "item")
        self.BTN_ENDPOINT_TRANSFER.setObjectName("BTN_ENDPOINT_TRANSFER")
        self.horizontalLayout_7.addWidget(self.BTN_ENDPOINT_TRANSFER)
        self.BTN_ENDPOINT_DISCONNECT = QtWidgets.QToolButton(self.frame)
        self.BTN_ENDPOINT_DISCONNECT.setMinimumSize(QtCore.QSize(96, 96))
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/bio42/disconnect.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_ENDPOINT_DISCONNECT.setIcon(icon5)
        self.BTN_ENDPOINT_DISCONNECT.setIconSize(QtCore.QSize(64, 64))
        self.BTN_ENDPOINT_DISCONNECT.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.BTN_ENDPOINT_DISCONNECT.setProperty("style", "item")
        self.BTN_ENDPOINT_DISCONNECT.setObjectName("BTN_ENDPOINT_DISCONNECT")
        self.horizontalLayout_7.addWidget(self.BTN_ENDPOINT_DISCONNECT)
        self.BTN_DB_STATUS = QtWidgets.QToolButton(self.frame)
        self.BTN_DB_STATUS.setMinimumSize(QtCore.QSize(96, 96))
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/bio42/select_database.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_DB_STATUS.setIcon(icon6)
        self.BTN_DB_STATUS.setIconSize(QtCore.QSize(64, 64))
        self.BTN_DB_STATUS.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.BTN_DB_STATUS.setProperty("style", "item")
        self.BTN_DB_STATUS.setObjectName("BTN_DB_STATUS")
        self.horizontalLayout_7.addWidget(self.BTN_DB_STATUS)
        self.BTN_OPTIMISE = QtWidgets.QToolButton(self.frame)
        self.BTN_OPTIMISE.setMinimumSize(QtCore.QSize(96, 96))
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/bio42/optimise.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_OPTIMISE.setIcon(icon7)
        self.BTN_OPTIMISE.setIconSize(QtCore.QSize(64, 64))
        self.BTN_OPTIMISE.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.BTN_OPTIMISE.setProperty("style", "item")
        self.BTN_OPTIMISE.setObjectName("BTN_OPTIMISE")
        self.horizontalLayout_7.addWidget(self.BTN_OPTIMISE)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_7.addItem(spacerItem1)
        self.verticalLayout_2.addLayout(self.horizontalLayout_7)
        self.gridLayout_3.addWidget(self.frame, 1, 0, 1, 1)
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_3.addItem(spacerItem2, 2, 0, 1, 1)
        self.label_8 = QtWidgets.QLabel(self.PAG_ENDPOINTS)
        self.label_8.setObjectName("label_8")
        self.gridLayout_3.addWidget(self.label_8, 0, 0, 1, 1)
        self.PAGER_MAIN.addTab(self.PAG_ENDPOINTS, "")
        self.PAG_IMPORT = QtWidgets.QWidget()
        self.PAG_IMPORT.setObjectName("PAG_IMPORT")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.PAG_IMPORT)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.frame_2 = QtWidgets.QFrame(self.PAG_IMPORT)
        self.frame_2.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame_2.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_2.setProperty("style", "contents")
        self.frame_2.setObjectName("frame_2")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.frame_2)
        self.verticalLayout.setObjectName("verticalLayout")
        self.label_5 = QtWidgets.QLabel(self.frame_2)
        self.label_5.setObjectName("label_5")
        self.verticalLayout.addWidget(self.label_5)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.BTN_IMPORT_SEQUENCES = QtWidgets.QToolButton(self.frame_2)
        self.BTN_IMPORT_SEQUENCES.setMinimumSize(QtCore.QSize(96, 96))
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(":/bio42/sequence.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_IMPORT_SEQUENCES.setIcon(icon8)
        self.BTN_IMPORT_SEQUENCES.setIconSize(QtCore.QSize(64, 64))
        self.BTN_IMPORT_SEQUENCES.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.BTN_IMPORT_SEQUENCES.setProperty("style", "item")
        self.BTN_IMPORT_SEQUENCES.setObjectName("BTN_IMPORT_SEQUENCES")
        self.horizontalLayout_3.addWidget(self.BTN_IMPORT_SEQUENCES)
        self.BTN_IMPORT_SIMILARITIES = QtWidgets.QToolButton(self.frame_2)
        self.BTN_IMPORT_SIMILARITIES.setMinimumSize(QtCore.QSize(96, 96))
        icon9 = QtGui.QIcon()
        icon9.addPixmap(QtGui.QPixmap(":/bio42/similarity.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_IMPORT_SIMILARITIES.setIcon(icon9)
        self.BTN_IMPORT_SIMILARITIES.setIconSize(QtCore.QSize(64, 64))
        self.BTN_IMPORT_SIMILARITIES.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.BTN_IMPORT_SIMILARITIES.setProperty("style", "item")
        self.BTN_IMPORT_SIMILARITIES.setObjectName("BTN_IMPORT_SIMILARITIES")
        self.horizontalLayout_3.addWidget(self.BTN_IMPORT_SIMILARITIES)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem3)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.label_6 = QtWidgets.QLabel(self.frame_2)
        self.label_6.setObjectName("label_6")
        self.verticalLayout.addWidget(self.label_6)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.BTN_IMPORT_TAXONOMY = QtWidgets.QToolButton(self.frame_2)
        self.BTN_IMPORT_TAXONOMY.setMinimumSize(QtCore.QSize(96, 96))
        icon10 = QtGui.QIcon()
        icon10.addPixmap(QtGui.QPixmap(":/bio42/taxonomy.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_IMPORT_TAXONOMY.setIcon(icon10)
        self.BTN_IMPORT_TAXONOMY.setIconSize(QtCore.QSize(64, 64))
        self.BTN_IMPORT_TAXONOMY.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.BTN_IMPORT_TAXONOMY.setProperty("style", "item")
        self.BTN_IMPORT_TAXONOMY.setObjectName("BTN_IMPORT_TAXONOMY")
        self.horizontalLayout_4.addWidget(self.BTN_IMPORT_TAXONOMY)
        self.BTN_IMPORT_ONTOLOGY = QtWidgets.QToolButton(self.frame_2)
        self.BTN_IMPORT_ONTOLOGY.setMinimumSize(QtCore.QSize(96, 96))
        icon11 = QtGui.QIcon()
        icon11.addPixmap(QtGui.QPixmap(":/bio42/go.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_IMPORT_ONTOLOGY.setIcon(icon11)
        self.BTN_IMPORT_ONTOLOGY.setIconSize(QtCore.QSize(64, 64))
        self.BTN_IMPORT_ONTOLOGY.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.BTN_IMPORT_ONTOLOGY.setProperty("style", "item")
        self.BTN_IMPORT_ONTOLOGY.setObjectName("BTN_IMPORT_ONTOLOGY")
        self.horizontalLayout_4.addWidget(self.BTN_IMPORT_ONTOLOGY)
        self.BTN_IMPORT_ANNOTATIONS = QtWidgets.QToolButton(self.frame_2)
        self.BTN_IMPORT_ANNOTATIONS.setMinimumSize(QtCore.QSize(96, 96))
        icon12 = QtGui.QIcon()
        icon12.addPixmap(QtGui.QPixmap(":/bio42/annotation.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_IMPORT_ANNOTATIONS.setIcon(icon12)
        self.BTN_IMPORT_ANNOTATIONS.setIconSize(QtCore.QSize(64, 64))
        self.BTN_IMPORT_ANNOTATIONS.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.BTN_IMPORT_ANNOTATIONS.setProperty("style", "item")
        self.BTN_IMPORT_ANNOTATIONS.setObjectName("BTN_IMPORT_ANNOTATIONS")
        self.horizontalLayout_4.addWidget(self.BTN_IMPORT_ANNOTATIONS)
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem4)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.label_7 = QtWidgets.QLabel(self.frame_2)
        self.label_7.setObjectName("label_7")
        self.verticalLayout.addWidget(self.label_7)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.BTN_IMPORT_CSV = QtWidgets.QToolButton(self.frame_2)
        self.BTN_IMPORT_CSV.setMinimumSize(QtCore.QSize(96, 96))
        icon13 = QtGui.QIcon()
        icon13.addPixmap(QtGui.QPixmap(":/bio42/csv.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_IMPORT_CSV.setIcon(icon13)
        self.BTN_IMPORT_CSV.setIconSize(QtCore.QSize(64, 64))
        self.BTN_IMPORT_CSV.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.BTN_IMPORT_CSV.setProperty("style", "item")
        self.BTN_IMPORT_CSV.setObjectName("BTN_IMPORT_CSV")
        self.horizontalLayout_8.addWidget(self.BTN_IMPORT_CSV)
        spacerItem5 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem5)
        self.verticalLayout.addLayout(self.horizontalLayout_8)
        self.gridLayout_4.addWidget(self.frame_2, 1, 0, 1, 1)
        spacerItem6 = QtWidgets.QSpacerItem(20, 619, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_4.addItem(spacerItem6, 2, 0, 1, 1)
        self.label_11 = QtWidgets.QLabel(self.PAG_IMPORT)
        self.label_11.setObjectName("label_11")
        self.gridLayout_4.addWidget(self.label_11, 0, 0, 1, 1)
        self.PAGER_MAIN.addTab(self.PAG_IMPORT, "")
        self.PAG_QUERY = QtWidgets.QWidget()
        self.PAG_QUERY.setObjectName("PAG_QUERY")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.PAG_QUERY)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.label_12 = QtWidgets.QLabel(self.PAG_QUERY)
        self.label_12.setObjectName("label_12")
        self.verticalLayout_8.addWidget(self.label_12)
        self.frame_3 = QtWidgets.QFrame(self.PAG_QUERY)
        self.frame_3.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame_3.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_3.setProperty("style", "contents")
        self.frame_3.setObjectName("frame_3")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.frame_3)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.label_9 = QtWidgets.QLabel(self.frame_3)
        self.label_9.setObjectName("label_9")
        self.verticalLayout_3.addWidget(self.label_9)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.BTN_QUERY_INBUILT = QtWidgets.QToolButton(self.frame_3)
        self.BTN_QUERY_INBUILT.setMinimumSize(QtCore.QSize(96, 96))
        self.BTN_QUERY_INBUILT.setMaximumSize(QtCore.QSize(96, 96))
        icon14 = QtGui.QIcon()
        icon14.addPixmap(QtGui.QPixmap(":/bio42/script.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_QUERY_INBUILT.setIcon(icon14)
        self.BTN_QUERY_INBUILT.setIconSize(QtCore.QSize(64, 64))
        self.BTN_QUERY_INBUILT.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.BTN_QUERY_INBUILT.setProperty("style", "item")
        self.BTN_QUERY_INBUILT.setObjectName("BTN_QUERY_INBUILT")
        self.horizontalLayout.addWidget(self.BTN_QUERY_INBUILT)
        spacerItem7 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem7)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.verticalLayout_8.addWidget(self.frame_3)
        self.frame_6 = QtWidgets.QFrame(self.PAG_QUERY)
        self.frame_6.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame_6.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_6.setProperty("style", "contents")
        self.frame_6.setObjectName("frame_6")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.frame_6)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.label_16 = QtWidgets.QLabel(self.frame_6)
        self.label_16.setObjectName("label_16")
        self.verticalLayout_7.addWidget(self.label_16)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.BTN_EXPORT_GEXF = QtWidgets.QToolButton(self.frame_6)
        self.BTN_EXPORT_GEXF.setMinimumSize(QtCore.QSize(96, 96))
        self.BTN_EXPORT_GEXF.setMaximumSize(QtCore.QSize(96, 96))
        icon15 = QtGui.QIcon()
        icon15.addPixmap(QtGui.QPixmap(":/bio42/network.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_EXPORT_GEXF.setIcon(icon15)
        self.BTN_EXPORT_GEXF.setIconSize(QtCore.QSize(64, 64))
        self.BTN_EXPORT_GEXF.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.BTN_EXPORT_GEXF.setProperty("style", "item")
        self.BTN_EXPORT_GEXF.setObjectName("BTN_EXPORT_GEXF")
        self.horizontalLayout_6.addWidget(self.BTN_EXPORT_GEXF)
        self.BTN_EXPORT_CSV = QtWidgets.QToolButton(self.frame_6)
        self.BTN_EXPORT_CSV.setMinimumSize(QtCore.QSize(96, 96))
        self.BTN_EXPORT_CSV.setMaximumSize(QtCore.QSize(96, 96))
        self.BTN_EXPORT_CSV.setIcon(icon13)
        self.BTN_EXPORT_CSV.setIconSize(QtCore.QSize(64, 64))
        self.BTN_EXPORT_CSV.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.BTN_EXPORT_CSV.setProperty("style", "item")
        self.BTN_EXPORT_CSV.setObjectName("BTN_EXPORT_CSV")
        self.horizontalLayout_6.addWidget(self.BTN_EXPORT_CSV)
        self.BTN_EXPORT_HTML = QtWidgets.QToolButton(self.frame_6)
        self.BTN_EXPORT_HTML.setMinimumSize(QtCore.QSize(96, 96))
        self.BTN_EXPORT_HTML.setMaximumSize(QtCore.QSize(96, 96))
        icon16 = QtGui.QIcon()
        icon16.addPixmap(QtGui.QPixmap(":/bio42/webpage.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_EXPORT_HTML.setIcon(icon16)
        self.BTN_EXPORT_HTML.setIconSize(QtCore.QSize(64, 64))
        self.BTN_EXPORT_HTML.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.BTN_EXPORT_HTML.setProperty("style", "item")
        self.BTN_EXPORT_HTML.setObjectName("BTN_EXPORT_HTML")
        self.horizontalLayout_6.addWidget(self.BTN_EXPORT_HTML)
        self.BTN_QUERY_PICKLE = QtWidgets.QToolButton(self.frame_6)
        self.BTN_QUERY_PICKLE.setMinimumSize(QtCore.QSize(96, 96))
        self.BTN_QUERY_PICKLE.setMaximumSize(QtCore.QSize(96, 96))
        icon17 = QtGui.QIcon()
        icon17.addPixmap(QtGui.QPixmap(":/bio42/pickle.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_QUERY_PICKLE.setIcon(icon17)
        self.BTN_QUERY_PICKLE.setIconSize(QtCore.QSize(64, 64))
        self.BTN_QUERY_PICKLE.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.BTN_QUERY_PICKLE.setProperty("style", "item")
        self.BTN_QUERY_PICKLE.setObjectName("BTN_QUERY_PICKLE")
        self.horizontalLayout_6.addWidget(self.BTN_QUERY_PICKLE)
        spacerItem8 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem8)
        self.verticalLayout_7.addLayout(self.horizontalLayout_6)
        self.verticalLayout_8.addWidget(self.frame_6)
        spacerItem9 = QtWidgets.QSpacerItem(20, 619, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_8.addItem(spacerItem9)
        self.PAGER_MAIN.addTab(self.PAG_QUERY, "")
        self.PAG_MISC = QtWidgets.QWidget()
        self.PAG_MISC.setObjectName("PAG_MISC")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.PAG_MISC)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.label_13 = QtWidgets.QLabel(self.PAG_MISC)
        self.label_13.setObjectName("label_13")
        self.gridLayout_6.addWidget(self.label_13, 0, 0, 1, 1)
        self.frame_4 = QtWidgets.QFrame(self.PAG_MISC)
        self.frame_4.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame_4.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_4.setProperty("style", "contents")
        self.frame_4.setObjectName("frame_4")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.frame_4)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.label_14 = QtWidgets.QLabel(self.frame_4)
        self.label_14.setObjectName("label_14")
        self.verticalLayout_4.addWidget(self.label_14)
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.BTN_HELP = QtWidgets.QToolButton(self.frame_4)
        self.BTN_HELP.setMinimumSize(QtCore.QSize(96, 96))
        icon18 = QtGui.QIcon()
        icon18.addPixmap(QtGui.QPixmap(":/bio42/help.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_HELP.setIcon(icon18)
        self.BTN_HELP.setIconSize(QtCore.QSize(64, 64))
        self.BTN_HELP.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.BTN_HELP.setProperty("style", "item")
        self.BTN_HELP.setObjectName("BTN_HELP")
        self.horizontalLayout_9.addWidget(self.BTN_HELP)
        self.BTN_INTERMAKE = QtWidgets.QToolButton(self.frame_4)
        self.BTN_INTERMAKE.setMinimumSize(QtCore.QSize(96, 96))
        icon19 = QtGui.QIcon()
        icon19.addPixmap(QtGui.QPixmap(":/intermake/intermake_logo.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_INTERMAKE.setIcon(icon19)
        self.BTN_INTERMAKE.setIconSize(QtCore.QSize(64, 64))
        self.BTN_INTERMAKE.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.BTN_INTERMAKE.setProperty("style", "item")
        self.BTN_INTERMAKE.setObjectName("BTN_INTERMAKE")
        self.horizontalLayout_9.addWidget(self.BTN_INTERMAKE)
        self.BTN_SETTINGS = QtWidgets.QToolButton(self.frame_4)
        self.BTN_SETTINGS.setMinimumSize(QtCore.QSize(96, 96))
        icon20 = QtGui.QIcon()
        icon20.addPixmap(QtGui.QPixmap(":/bio42/settings.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.BTN_SETTINGS.setIcon(icon20)
        self.BTN_SETTINGS.setIconSize(QtCore.QSize(64, 64))
        self.BTN_SETTINGS.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.BTN_SETTINGS.setProperty("style", "item")
        self.BTN_SETTINGS.setObjectName("BTN_SETTINGS")
        self.horizontalLayout_9.addWidget(self.BTN_SETTINGS)
        spacerItem10 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_9.addItem(spacerItem10)
        self.verticalLayout_4.addLayout(self.horizontalLayout_9)
        self.gridLayout_6.addWidget(self.frame_4, 1, 0, 1, 1)
        spacerItem11 = QtWidgets.QSpacerItem(20, 619, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_6.addItem(spacerItem11, 2, 0, 1, 1)
        self.PAGER_MAIN.addTab(self.PAG_MISC, "")
        self.gridLayout.addWidget(self.PAGER_MAIN, 0, 0, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1521, 22))
        self.menubar.setObjectName("menubar")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.retranslateUi(MainWindow)
        self.PAGER_MAIN.setCurrentIndex(4)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow"))
        self.LBL_INTRO.setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:xx-large; font-weight:600;\">B42</span></p><p><span style=\" font-style:italic;\">Version 0.0.0.0</span></p><p><span style=\" font-size:x-large; font-weight:600;\">Meta-graph</span></p><p><img src=\":/bio42/whiteboard.png\"/><br/></p><ul style=\"margin-top: 0px; margin-bottom: 0px; margin-left: 0px; margin-right: 0px; -qt-list-indent: 1;\"><li style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:large;\">The </span><a href=\"action:getting started\"><span style=\" font-size:large; font-weight:600; text-decoration: underline; color:#0000ff;\">getting started</span></a><span style=\" font-size:large;\"> window can help you begin!</span></li></ul></body></html>"))
        self.PAGER_MAIN.setTabText(self.PAGER_MAIN.indexOf(self.PAG_INTRO), _translate("MainWindow", "Introduction"))
        self.label_2.setText(_translate("MainWindow", "Open"))
        self.label_2.setProperty("style", _translate("MainWindow", "heading"))
        self.BTN_WIZARD.setStatusTip(_translate("MainWindow", "Connect to a Neo4j database."))
        self.BTN_WIZARD.setText(_translate("MainWindow", "Wizard"))
        self.BTN_ENDPOINT_DATABASE.setStatusTip(_translate("MainWindow", "Connect to a Neo4j database."))
        self.BTN_ENDPOINT_DATABASE.setText(_translate("MainWindow", "Server"))
        self.BTN_ENDPOINT_PACKAGE.setStatusTip(_translate("MainWindow", "Open a folder on disk. You can use this to prepare bulk data transfers or if you can\'t currently connect toa  a Neo4j database."))
        self.BTN_ENDPOINT_PACKAGE.setText(_translate("MainWindow", "Parcel"))
        self.BTN_ENDPOINT_MGRAPH.setStatusTip(_translate("MainWindow", "An MGraph file can be exported to a variety of outputs."))
        self.BTN_ENDPOINT_MGRAPH.setText(_translate("MainWindow", "File"))
        self.label_4.setText(_translate("MainWindow", "Maintain"))
        self.label_4.setProperty("style", _translate("MainWindow", "heading"))
        self.BTN_ENDPOINT_TRANSFER.setStatusTip(_translate("MainWindow", "Transfer data from one endpoint to another."))
        self.BTN_ENDPOINT_TRANSFER.setText(_translate("MainWindow", "Transfer"))
        self.BTN_ENDPOINT_DISCONNECT.setStatusTip(_translate("MainWindow", "Disconnect an endpoint. This removed the endpoint from BIO42 but does not remove the data from disk - you can always reopen the endpoint later."))
        self.BTN_ENDPOINT_DISCONNECT.setText(_translate("MainWindow", "Disconnect"))
        self.BTN_DB_STATUS.setStatusTip(_translate("MainWindow", "Connect to a Neo4j database."))
        self.BTN_DB_STATUS.setText(_translate("MainWindow", "Database"))
        self.BTN_OPTIMISE.setStatusTip(_translate("MainWindow", "Connect to a Neo4j database."))
        self.BTN_OPTIMISE.setText(_translate("MainWindow", "Optimize"))
        self.label_8.setText(_translate("MainWindow", "Open an endpoint so you have somewhere to send your data."))
        self.label_8.setProperty("style", _translate("MainWindow", "helpbox"))
        self.PAGER_MAIN.setTabText(self.PAGER_MAIN.indexOf(self.PAG_ENDPOINTS), _translate("MainWindow", "Endpoints"))
        self.label_5.setText(_translate("MainWindow", "Genomics"))
        self.label_5.setProperty("style", _translate("MainWindow", "heading"))
        self.BTN_IMPORT_SEQUENCES.setText(_translate("MainWindow", "Sequences"))
        self.BTN_IMPORT_SIMILARITIES.setText(_translate("MainWindow", "Similarities"))
        self.label_6.setText(_translate("MainWindow", "Classification"))
        self.label_6.setProperty("style", _translate("MainWindow", "heading"))
        self.BTN_IMPORT_TAXONOMY.setText(_translate("MainWindow", "Taxonomy"))
        self.BTN_IMPORT_ONTOLOGY.setText(_translate("MainWindow", "Ontology"))
        self.BTN_IMPORT_ANNOTATIONS.setText(_translate("MainWindow", "Annotations"))
        self.label_7.setText(_translate("MainWindow", "Miscellaneous"))
        self.label_7.setProperty("style", _translate("MainWindow", "heading"))
        self.BTN_IMPORT_CSV.setText(_translate("MainWindow", "Custom"))
        self.label_11.setText(_translate("MainWindow", "You can import data to any endpoint that you have defined."))
        self.label_11.setProperty("style", _translate("MainWindow", "helpbox"))
        self.PAGER_MAIN.setTabText(self.PAGER_MAIN.indexOf(self.PAG_IMPORT), _translate("MainWindow", "Import"))
        self.label_12.setText(_translate("MainWindow", "Once you have data in your database you can run queries and export to a variety of formats."))
        self.label_12.setProperty("style", _translate("MainWindow", "helpbox"))
        self.label_9.setText(_translate("MainWindow", "Queries"))
        self.label_9.setProperty("style", _translate("MainWindow", "heading"))
        self.BTN_QUERY_INBUILT.setText(_translate("MainWindow", "Query"))
        self.label_16.setText(_translate("MainWindow", "Export"))
        self.label_16.setProperty("style", _translate("MainWindow", "heading"))
        self.BTN_EXPORT_GEXF.setToolTip(_translate("MainWindow", "Export to .GEXF"))
        self.BTN_EXPORT_GEXF.setText(_translate("MainWindow", "Gephi"))
        self.BTN_EXPORT_CSV.setToolTip(_translate("MainWindow", "Export to .CSV"))
        self.BTN_EXPORT_CSV.setText(_translate("MainWindow", "Excel"))
        self.BTN_EXPORT_HTML.setToolTip(_translate("MainWindow", "Export to .HTML"))
        self.BTN_EXPORT_HTML.setText(_translate("MainWindow", "Interative\n"
"web page"))
        self.BTN_QUERY_PICKLE.setToolTip(_translate("MainWindow", "Export to .PICKLE"))
        self.BTN_QUERY_PICKLE.setText(_translate("MainWindow", "Python"))
        self.PAGER_MAIN.setTabText(self.PAGER_MAIN.indexOf(self.PAG_QUERY), _translate("MainWindow", "Query"))
        self.label_13.setText(_translate("MainWindow", "Features that don\'t fall into any other category."))
        self.label_13.setProperty("style", _translate("MainWindow", "helpbox"))
        self.label_14.setText(_translate("MainWindow", "Miscellaneous"))
        self.label_14.setProperty("style", _translate("MainWindow", "heading"))
        self.BTN_HELP.setText(_translate("MainWindow", "Help"))
        self.BTN_INTERMAKE.setText(_translate("MainWindow", "Intermake"))
        self.BTN_SETTINGS.setText(_translate("MainWindow", "Settings"))
        self.PAGER_MAIN.setTabText(self.PAGER_MAIN.indexOf(self.PAG_MISC), _translate("MainWindow", "Miscellaneous"))


